/* ==================================================== ======== ======= *
 *
 *  uappli.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uappli_hh
#define	_uappli_hh
//pragma ident	"@(#)uappli.hh	ubit:2.11.0"
#include <udisp.hh>
#include <ugroup.hh>
#include <uprop.hh>
#include <uevent.hh>
#include <uconf.hh>


/** Application Context. notes:
 * - there should be only one application context per program.
 * - check methods inherited from UDisp, the Display Context
 */
class UAppli: public UDisp {
public:
  static UConf conf;
  ///< configuration of the UAppli (see UConf).

  UAppli(int* argc, char* argv[], const char *display_name = null);
  /**<
   * standard constructor for creating the Application Context.
   * Arguments of the constructor:
   * - 'argc' and 'argv' : the parameters of the main() function
   * - 'display_name' : the X server (default = null means localhost)
   *
   * Arguments of the command line:
   * - the X server and the config profile can be set in the following way:
   *         myappli -prof IPAQ -display myserver:0 
   * - other options: 
   *     -sync[hronous], -dual_mouse, -double_buffering, -soft_windows
   *
   * Notes:
   * - the config profile can also be set by function setProfile()
   *
   * - applications will use True Color mode when possible because certain
   *   features (such as translucency) are nore available with Pseudo Colors.
   *   See: UDisp::setTrueColors() and UDisp::setPseudoColors() 
   *   for details on True and Pseudo Color modes and X visuals.
   */

  virtual ~UAppli();

  bool setProfile(const char* profile);
  bool setProfile(class UConf&);
  ///< sets the config profile (see UConf::setProfile()).

  static UAppli& getDefaultAppli();
  ///< gets the defaut appli (as there is usually only one UAppli).

  void setTitle(const UStr &title);
  ///< changes the title of the Main Frame (if any: the main frame is the first UFrame that is added to the UAppli.

  virtual void realize();
  /**< initializes X data.
   * this function does not need to be explicitely called except if you try
   * to draw Graphics before calling the mainLoop 
   */

  virtual bool isRealized() const;
  ///< is the X data initialized?.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // Windows

  virtual void add(class UWin*);
  virtual void add(class UWin&);
  /**< 
   * method for adding windows to the UAppli.
   * important notes:
   * - at least one Window (typically an UFrame) should be added to
   *   the UAppli instance
   * - the "Main Frame" is the first UFrame that is added to the UAppli
   * - windows are initially closed (including the Main Frame) :
   *   you must first add their subcomponents (using add(), addlist() etc.)
   *   THEN open them (when needed) by calling their UWin::show() method.
   * - by default their size won't change once their show() method has
   *   been called.
   * See classes: UWin, UFrame, UDialog.
   */ 

  virtual UWin* remove(UWin& child, bool auto_delete);
  virtual UWin* remove(UWin* child, bool auto_delete);
  ///< same as UGroup::remove().

  class UFrame* getMainFrame() const;
  ///< returns the Main Frame (if any: the main frame is the first UFrame that is added to the UAppli.

  class UGroup* getOpenedMenu() const;
  ///< returns the menu that is currently opened (null otherwise).

  virtual void updateAll();
  virtual void updateAll(UUpdate upmode); 
  ///< updates all windows (see UWin::update()).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // inputs and timers

  virtual void add(class UInput*);
  ///< adds a UInput object that activates callbacks when a file or a socket gets data.

  virtual void add(class UTimer*);
  ///< adds a UTimer object that activates callbacks when a timeout is reached (or when the main loop become idle).

  virtual class UInput* remove(class UInput*, bool auto_delete);
  virtual class UTimer* remove(class UTimer*, bool auto_delete);
  ///< works in the same ways as UGroup::remove().

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // UMS: dual mouse (bi-manual interaction), etc.

  bool openUMS(const char *ums_host = null, int ums_port = 0);
  /**< opens the UMS (Ubit Message Server) for bi-manual interaction, etc.
   *  Args:
   *  - ums_host: the host where the UMS is running; null means local host
   *  - ums_port: 0 for the default UMS port (which is 45455)
   *  Notes: 
   *  - the UMS must already be running (ie. the 'ums' program)
   *  - this function is automatically called if the UAppli is launched
   *    in dual mouse mode (see UAppli::UAppli and UConf) 
   */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // main loop

  virtual int mainLoop();
  /**< starts the event main loop of the application.
   * returns the status argument given as an arg of the quit() method */

  virtual void quitLoop(int status);
  static void quit(int status);
  /**< quits the main loop of the application.
   * Notes:
   * - this status is returned by the mainLoop() method (it should typically
   *   be 0 for normal termination and > 0 otherwise)
   * - this method can be redefined by clients for appropriate behaviors
   *   (eg. saving data before quitting etc.) */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // variable database

  const char* getVar(const char *name, bool get_from_shell = true);
  /**< retreives a value from a variable name.
   * searches variable in the Application variable database, then, if not
   * found and 'get_from_shell' is true, in the Unix SHELL environment.
   * - note: when a variable is retrieved from the SHELL, it is copied in
   *   the application database and is never searched again in the SHELL */

  const char* getImaPath();
  /**< gets the value of the UIMA_PATH variable (the DEFAULT location of images).
   * see: setImaPath() and getVar() */

  const char* setVar(const char *name, const char *value);
  /**< adds a variable/value pair to the Application variable database.
   * Notes:
   * - the 'name' and 'value' strings are internally duplicated.
   * - this fct. returns the duplicated value (null if memory is exhausted)
   * - variables can be used for defining image file prefixes:
   *   uima("$MYDIR/my_image.gif") => MYDIR is expanded to its actual value
   * - see: setImaPath() (and special variable UIMA_PATH) */

  const char* setImaPath(const UStr& value);
  const char* setImaPath(const char* value);
  /**< sets the value of the UIMA_PATH variable (the DEFAULT location of images).
   * the value of the UIMA_PATH variable is prefixed to image file names 
   * that do not start with / or . or $
   * <br>Examples:
   * - uima("my_image.gif")        will load:  $UIMA_PATH/my_image.gif
   * - uima("./my_image.gif")      will load:  ./my_image.gif
   * - uima("$MYDIR/my_image.gif") will load:  $MYDIR/my_image.gif 
   * where $XXX is replaced by the value of variable XXX */

  char *makeImaPath(const char* filename);
  /**< creates a full image file path name.
   * prefixes 'filename' with UIMA_PATH value if not starting by / . or $
   * and expands 'filename' if starting by $  */


  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  class UInputFlow& getDefaultFlow() const;
  class UInputFlow* getFlow(class UEvent&) const;
  ///< [impl] returns the corresponding Input Flow

  class UNatAppli* getNatAppli() const {return (UNatAppli*)natdisp;}
  ///< [impl] returns the Native Appli object (implementation specific data).

  unsigned int getDispCount() const;
  unsigned int getFlowCount() const;

#ifndef NO_DOC
  friend class UDisp;
  friend class UInputFlow;
  friend class UNatAppli;

  void closeRelatedMenus(class UMenu*);

  void deleteNotify(class UView* deleted_view);
  void deleteNotify(class UGroup* deleted_group);
  static void deleteNotifyAll(class UGroup* deleted_group);
  ///< [impl] notifies the application(s) that a view has been deleted.

  int getModalStatus() const  {return modal_status;}
  void setModalStatus(int _s) {modal_status = _s;}
  const char* getAppliName() const {return appli_name;}

  virtual int subLoop();   ///< [impl] subloop lauched by modal dialogs
  void addModalwin(class UWin*);
  void removeModalwin(class UWin*);

private:  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  static UAppli* default_appli;

  std::vector<UDisp*> displist;	// list of displays
  std::vector<UInputFlow*> flowlist;	// list of input flows

  class UGroup *winlist, *inputs, *timers;      // window and callback lists

  const char *pixmapPath, *bitmapPath;  // image paths
  const char *appli_name;
  struct UAPP_VAR* varDB;	        // variable DataBase array
  int varDB_count;                      // and it cardinal

  class UFrame* main_frame;	// the main frame of the appli

  // current modal dialog in the Display
  // (null if no modal window currently activated)
  UChain modalwin_list;
  class UWin* modalwin;

  enum {LOOP_STOP = -2, LOOP_RUN = -1, LOOP_QUIT = 0};
  // value is init. to LOOP_IDLE by UAppli(), then to LOOP_RUN
  // by mainLoop() and a value >= LOOP_QUIT quits the mainLoop
  int main_status;
  // used by modal dialogs for controlling their modalloop
  int modal_status;
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** a UInput object activates callbacks when a file or a socket gets data.
 *  Usage:
 *  - this object must be "added" to the UAppli to become active
 *    (see: UAppli::add(UInput&) and UAppli::remove(UInput&, bool))
 *  - callback functions and methods are added (or removed) in the usual way
 *    to this object (see: UGroup::add(), UGroup::addlist(), UGroup::remove())
 */
class UInput : public UGroup {
public:
  UInput(int source);
  ///< constructor: 'source' must be a valid file or socket ID.

  virtual ~UInput();

  int getSource() const {return source;}

protected:
  virtual bool verifies(const UContext*, const UCtrl*) const {return false;}

private:
  friend class UNatAppli;
  int source;
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** a UTimer object activates callbacks when a timeout is reached 
 *  (or when the main loop become idle).
 *  Usage:
 *  - this object must be "added" to the UAppli to become active
 *    (see: UAppli::add(UTimer&) and UAppli::remove(UTimer&, bool))
 *  - callback functions and methods are added (or removed) in the usual way
 *    to this object (see: UGroup::add(), UGroup::addlist(), UGroup::remove())
 */
class UTimer : public UGroup {
public:
  UTimer(unsigned long delay, int ntimes, bool self_delete);
  /**< constructor.
   * Arguments:
   * - 'ntimes': how many times the timer is executed. -1 means always, 0 never.
   * - 'delay' is in milliseconds. 
   * - if 'delay' = 0, the callback is immediately called when the main loop 
   *   becomes idle. This is useful for postponing callbacks that take a long
   *   time to execute. 'ntimes' should usually be 1 in this case (but other
   *   values are accepted; a value of -1 means that callbacks will be always
   *   called when the main loop becomes idle)
   * - 'self_delete' means that this object is automatically destroyed and
   *   removed from the UAppli when over (be careful if you point such objects:
   *   use Ubit References (see template uref)
   */

  virtual ~UTimer();

  virtual void reset(unsigned long _delay, int _ntimes);

  int getDelay() const {return delay;}
  int getTimes() const {return ntimes;}

protected:
  virtual bool verifies(const UContext*, const UCtrl*) const {return false;}

private:
  friend class UAppli;
  friend class UNatAppli;

  unsigned long delay;
  int  ntimes;
  bool must_rearm, self_del;
  struct timeval* timeout;
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
