/* ==================================================== ======== ======= *
 *
 *  ubox.hh : Ubit Box containers.
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _ubox_hh
#define	_ubox_hh
//pragma ident	"@(#)ubox.hh	ubit:2.11.0"
#include <ugroup.hh>
#include <uboxImpl.hh>


/** Graphical "viewable" container.
 *
 *  This class is a lightweight container that is able to control one 
 *  or several Views (UView class) on the Screen. This class is the base
 *  for all interactive "gadgets" (UButton, etc.), visible containers 
 *  (UHbox, UVbox, UMenubar, etc.) and Windows (UWin, UDialog, UMenu...)
 * 
 *  See:
 *  - UGroup::~UGroup for <b>important notes about deletion</b>
 *  - UCtrl (the generic controller) and UGroup as most methods are 
 *    inherited from these classes
 *  - various gadgets and interactors (ULabel, UButton, UTextbox, etc.)
 *    that are derived from this class
 *  - UWin (base class for creating menus ans dialogs) that is also
 *    derived from this class
 *
 *  Geometry:
 *  - UBox, UHbox, UBar : horizontal lay out by default (add brick: 
 *    UOrient::vertical as a child for vertical lay out). These objects
 *    are "flexible" in the vertical direction: children are automatically 
 *    adjusted in this direction (same as uvflex())
 *
 *  - UVbox : vertical lay out by default. This object is "flexible" 
 *    in the horizontal direction: children are automatically adjusted 
 *    in this direction (same as uhflex())
 *
 *  - UFlowbox : "flow" lay out (similar to those of an HTML page)
 *    Typically used from creating text areas or hypermedia gadgets.
 *
 *  - Boxes are resized when their children are modified (ie. added,
 *    removed or resized). 
 *
 *  - UFlowbox implements a specific behavior: its height changes
 *    but its width remains constant (see this class for details)
 *
 *  - UFlowbox (and UTextbox and subclasses) should be used for 
 *    displaying textual messages that are dynamically changed.
 *
 * Examples:
 *  <pre><tt>
 *     UBox& r = uhbox( UFont::bold + "String" + ubutton("Button") + etc. );
 *     UBox* p = &uvbox( UColor::red + ubutton(UPix::folder + "Open") + etc. );
 *     uref<UBox>smp = ubar( ubutton("Btn 1") + ubutton("Btn 2") + etc. );
 * </tt></pre>
 *
 * These 3 examples show how to create objects in the heap (the dynamic memory). 
 * uhbox(), uvbox(), etc. are <b>creator shortcuts</b> that create a new
 * object by calling primitive 'new':  uhbox(...)  equals  *new UHbox(...) 
 * They can have a list of child objects as an argument (these children being
 * separated by the + operator). 
 *
 * Notes:
 * - uref<UBox> is a <b>Ubit Reference</b> (a smart pointer that is somewhat 
 *   similar to a Java references). uref(s) makes it possible to handle objects 
 *   created in the heap (= by using new) in a safe way. This is not the case
 *   with usual C++ pointers and references (especially when objects are deleted).
 *   See: class UBrick and template uref for more info.
 *
 * - uhbox("String") is equivalent to: uhbox(ustr("String"))
 * - uhbox("String1" + "String2") is illegal (that's a C++ limitation).
 *   instead you shouls write:  uhbox(ustr("String1") + "String2")
 * - strings can be made changeable and/or editable as follows:
 *  <pre><tt>
 *     UStr& str = ustr("String")
 *     UBox& box = uhbox(uedit() + str);  // uedit() makes string editable
 *     ....
 *     cout << str + endl;   // prints the content of 'str'
 *     str.set("xxxxx");     // changes the content of 'str' and update 'box'
 */
class UBox: public UGroup {
public:
  static UStyle *style;

  UBox(const UArgs& a = UArgs::none);
  /**< 
   * constructor; see also class UGroup, destructor UGroup::~UGroup() and the <em>creator shortcut</em> <b>ubox()</b>.
   */

  friend UBox& ubox(const UArgs& a = UArgs::none) {return *new UBox(a);}
  /**< 
   * creator shortcut that is equivalent to: *new UBox().
   * Note: watch the case: UBox is a class while ubox is a function!
   */

  virtual ~UBox() {cleanup();}
  ///< see <b>important notes</b> on <b>recursive deletion</b> : see UGroup::~UGroup()

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static  const UStyle& makeStyle();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // methods (same as UGroup counterparts)

  UBox& addlist(const UArgs& a) {UGroup::addlist(a); return *this;}

  virtual void update();
  virtual void update(UUpdate upmode);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // view management

  virtual UView*  getView(int no) const;
  virtual UView** getViews() const;
  virtual UView** getViews(int &view_count) const;
  virtual int getViewCount() const;
  /**<
   * get the views managed by this Box.
   * these functions work in the same way as UGroup::getChidren()/getChild() */

  /* OBS: class UViewIndex getViewIndex() const;
   * < optimized API for accessing Views.
   * this API is implementation dependant. see: UViewIndex in ubrickImpl.hh */

  UView* getViewContaining(UEvent&) const;
  ///< returns the Box's View that contains the Event location (null if none).

  UView* getViewContaining(UView *child_view) const;
  ///< returns the Box's View that contains this (direct or indirect) child view (null if none).

  UView* getFirstViewInside(UView* parent_view) const;
  /**< 
   * returns the first Box's View that is contained in this (direct) parent view.
   * Notes:
   * - 'parent_view' must be a DIRECT parent
   * - there will be several Box's views inside 'parent_view' if the Box has beed 
   *   multiply added to this parent */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // box events

  virtual UBox& onPostChildEvent(const UArgs&);
  /**<
   * gets events that are processed by children (and descendants).
   * Details:
   * - events are first sent to children, then to this object
   * - events which are not deal with by children are lost
   * - all UEvent methods (such as getSource(), getX(), getY() ...)
   *   are relative to the CHILD which received this event first
    * <pre>Example:
   *   box->onPostChildEvents( UOn::mmove :: ucall(arg1, func1) 
   *                          + UOn::mpress:: ucall(arg2, func2))
   * </pre>
   * See also event condition: UOn::preChildEvent  which is used to detects 
   * events *before* they are sent to children
  */

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

public:
  friend class UWin;

  virtual class UBox* boxCast() {return this;}
  virtual ULink* makeLink() {return new UBoxLink(this);}
  virtual void init(ULink *selflink, UView *ancestor_view);
  virtual void initAddedChild(UGroup*, ULink *childlink);
  virtual void updateView(UEvent&, UView*, const UUpdate&);
  static void updateImpl(const class UUpdate& upmode, 
			 UView* winview, UView* layoutview, UView* showview,
			 bool impose_size, u_dim w, u_dim h); 
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** UVbox = vertical Box.
 *  Geometry: see class UBox for details.
 */
class UVbox: public UBox {
public:
  static UStyle *style;

  UVbox(const UArgs& a = UArgs::none) : UBox(a) {}
  ///< constructor; see also ~UGroup and the <em>creator shortcut</em> <b>uvbox()</b>

  friend UVbox& uvbox(const UArgs& a = UArgs::none) {return *new UVbox(a);}
  ///< creator shortcut that is equivalent to: *new UVbox()

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static const UStyle& makeStyle();
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** UHbox = horizontal Box.
 *  Geometry: see class UBox for details.
 */
class UHbox: public UBox {
public:
  static UStyle *style;

  UHbox(const UArgs& a = UArgs::none) : UBox(a) {}
  ///< constructor; see also ~UGroup and the <em>creator shortcut</em> <b>uhbox()</b>

  friend UHbox& uhbox(const UArgs& a = UArgs::none) {return *new UHbox(a);}
  /// creator shortcut that is equivalent to: *new UHbox().

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static const  UStyle& makeStyle();
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** UBar = horizontal Bar (for making toolbars).
 *  This class is similar to UHbox but with a specific ("toolbar like") decoration.
 *  Buttons included in this object will also be displayed in a specific
 *  way (their border won't appear, etc.)
 *
 *  Geometry: see class UBox for details.
 *
 *  See also: classes UMenu and UMenubar for creating menus and menu bars
 */
class UBar: public UBox {
public:
  static UStyle *style;

  UBar(const UArgs& a = UArgs::none): UBox(a) {}
  ///< constructor; see also ~UGroup and the <em>creator shortcut</em> <b>ubar()</b>

  friend UBar& ubar(const UArgs& a = UArgs::none) {return *new UBar(a);}
  ///< creator shortcut that is equivalent to: *new UBar()

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static  const UStyle& makeStyle();
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** UFlowbox = Box with a Flow Layout (similar to an HTML page).
 *  This class is typically used from creating text areas or hypermedia gadgets.
 *
 *  Geometry: this class implements a specific behavior that differs from
 *  other boxes: its height changes but its width remains constant when 
 *  children are modified (= these objects will keep their initial width).
 *  The width can be specified by adding an UWidth child. It can also
 *  be changed through user interaction if this object is included
 *  in a box that is "flexible" in the horizontal direction (see UBox).
 *
 *  UFlowbox (and UTextbox and subclasses) should be used for displaying 
 *  textual messages that are dynamically changed for optimization purpose.
 *
 *  an UFlowbox is an UBox with UFlowView renderer.
 */
class UFlowbox: public UBox {
public:
  static UStyle *style;

  UFlowbox(const UArgs& a = UArgs::none);

  friend UFlowbox& uflowbox(const UArgs& a = UArgs::none)
  {return *(new UFlowbox(a));}

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static  const UStyle& makeStyle();
};


#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
