/* ==================================================== ======== ======= *
 *
 *  ubrick.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _ubrick_hh
#define	_ubrick_hh
//pragma ident	"@(#)ubrick.hh	ubit:2.11.0"
#include <typeinfo>
#include <stdlib.h>  // for size_t, new(), delete()
#include <udefs.hh>
#include <ubrickImpl.hh>

/** UClassID : Ubit Class ID
 *
 *  Each class has an unique ID (see: UBrick::classID(), UBrick::getClassID()).
 *  This ID is used for identifying classes and class/instance relationships
 *  (see: UBrick::isClassInstance(), UBrick::isInstance(), UGroup::getChild()).
 *  IDs values won't necessarily be the same in different programs.
 */
typedef bool (*UClassID)(const UBrick*);


/** UClass template
 */
template <class CC>
class UClass {
public:
  static bool isInstance(const UBrick*_obj) {return dynamic_cast<const CC*>(_obj);}
  static UClassID getID() {return isInstance;}
};


/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Ubit Brick: base class for objects that can be added to the 
 *  instance graph (abstract class).
 *
 *  See IMPORTANT NOTES:
 *  -1) on memory management : see example and remarks below
 *  -2) on <b>recursive deletion</b> : see destructor UBrick::~UBrick
 *  -3) on <b>Ubit References</b> (smart pointers) : see template uref
 *  -4) on dynamic type identification : see type UClassID and template UClass
 *
 *  This class forbids assignment between objects (the = operator being 
 *  dangerous and nonsensical in the general case).
 *  However, certain subclasses define specific = operators that can
 *  be used safely (for instance UStr, UColor, UFont). By opposition
 *  to Java, these = operators always copy the objects' content (or, more
 *  precisely, the subpart of these objects that it makes sense to copy).
 *
 *  Examples:
 *  <pre><tt>
 *  void foo() {
 *                        // suppose XX is a non abstract UBrick subclass
 *    XX  obj1, obj2;     // objects allocated in the stack [watch! :see #1]
 *    XX* p1 = new XX();  // C++ pointer to an object allocated in the heap [see #2]
 *    XX& r1 = *new XX(); // C++ reference to an object allocated in the heap [see #2]
 *    XX& r2;             // ILLEGAL: C++ refs must be initialized.
 *
 *    p1 = &obj1;         // p1 points to obj1 [watch!: see #2 and #3]
 *    XX& r2 = obj2;      // r2 points to obj2 [watch!: see #2 and #3]
 *    p1 = &r2;           // p1 points to obj2 [watch!: see #2 and #3]
 *
 *    obj2 = obj1;        // ILLEGAL for most subclasses (wont' compile)
 *                        // CORRECT for UStr, UColor, UFont... : copies content
 *
 *    r2 = obj1;          // same as: obj2 = obj1 : ILLEGAL in the general case
 *                        // COPIES objects' content otherwise
 *
 *    delete p1;          // VERY UNSAFE in the general case [see #4]
 *                        // WRONG here (p1 points to obj2 which is in the stack)
 *
 *    uref<XX> smp = new XX(); // what SHOULD BE DONE (Ubit References) [see #5]
 *    smp = null;         // the only safe way to destroy an object [see #5]
 *
 * </tt></pre>
 * 
 * Important remarks:
 *
 * - #1 : allocating objects in the stack is generally inappropriate: these
 *        objects will be destroyed when the function returns. This should 
 *        only be used for temporary data.
 *
 * - #2 : C++ pointers and references are unsafe and should be used with great 
 *        care: they won't be notified if the object they point to is deleted.
 *        Hence, they should only be used locally in functions if you are sure
 *        that objects won't be deleted. 
 *
 *        <b>USE UBIT REFERENCES (see: #5) IN ALL OTHER CASES!</b>
 *
 * - #3 : these C++ pointers and references points to objects that were
 *        allocated in the stack (see #1). Hence, they must remain LOCAL to
 *        the function body: they cannot be returned by the function because
 *        they would point to undefined data.
 *
 * - #4 : the 'delete' primitive is VERY UNSAFE for a bunch of reasons
 *        (see: UBrick::~UBrick() for explanations and methods for destroying
 *        objects safely). For instance, it will try to destroy data that was
 *        allocated in the stack in this example (this will produce an error).
 *
 * - #5 : Ubit references are "smart pointers" that are somewhat similar to
 *        Java references.  They makes it possible to handle objects created 
 *        in the heap (= by using new) in a safe way.
 *
 *        See: template uref for details.
 */

class UBrick {  
  UBrick(const UBrick&);
  UBrick& operator=(const UBrick&);
  ///< assigment is forbidden (except for certain subclasses) see class UBrick

public:

  UBrick(u_modes b_modes = 0);
  /**< 
   * standard constructor.
   * argument 'bmode' is used internally and should not be set by clients */

  virtual ~UBrick();
  /**<
   * see <b>important notes</b> on <b>recursive deletion</b>.
   * All destructors <b>destroys the children</b> and descendants
   * of the object being deleted if:
   * - they do not have other (undeleted) parents, and
   * - they are not referenced by an uref (Ubit Reference), and
   * - they have been created in the heap (ie. by using the 'new' primivite
   *   or a 'creator shortcut' such as ugroup())
   *
   * <b>Advices:</b>
   * - avoid using the <b>delete</b> primivite and prefer the
   *   UGroup::remove() or UGroup::removeAll() functions
   *
   * - 'delete' does not update graphics and may produce unexpected 
   *   results or generate errors because objects:
   *   -# can be shared (= they can have several parents)
   *   -# can be created in the static memory or in the stack
   *   -# can be refrenced by uref's (Ubit References)
   *  deleting objects is illegal in the two later cases and will 
   *  generate an error or an exception.
   *
   * - See also: UGroup::remove(), UGroup::removeAll(), and uref
   */

  virtual const char* getClassName() const {return typeid(*this).name();}
  ///< returns the name of the class of <i>this</i> object.

  void setAutoUpdate(bool);
  ///< specifies whether update() is invoked when set(), add() or remove() are called.

  bool isAutoUpdate() const;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // access to parents

  virtual int getParentCount() const;
  /**< returns the number of parents. */

  virtual UGroup** getParents() const;
  virtual UGroup** getParents(int &parent_count) const;
  /**< 
   * returns a copy of the object's parent list.
   * notes:
   * - returns null if the object has no parent
   * - returns the parent list if the object has at least one parent
   * - this list must be destroyed by client by calling:  delete[] this_list; 
   * - Warning: Ubit objects can have several parents! 
   */

  virtual int getParents(std::vector<UGroup*>& parent_vect) const;
  /**< 
   * returns a copy of the object's parent list.
   * returns the number of parents and intializes the vector of parents.
   * Note: parent_vect is not cleared : parents are added to the 
   *       parent_vect argument if its is not empty
   * See also: UGroup** getParents() and getParent().
   */

  virtual UGroup* getParent(int pos) const;
  /**< 
   * returns the nth parent.
   * note that brick objects (and subclasses) can have several parents
   * - 'pos' = 0 means "first parent" and 'pos' = -1 means "last parent"
   * - returns null and issues a warning message if 'pos' is out of range
   * - use getParents() is you need to access several parents:
   *   this will be more efficient than multiple calls to getParent() */

  virtual bool isChildOf(class UGroup *possible_parent, bool indirect) const;
  /**<
   * is this object a direct (resp. indirect) child of the object given as
   * an argument?.
   * second argument specifies if we accept indirect childhood (indirect' 
   * is true) or if we just consider direct children ('indirect' is false) */

  virtual void removeFromParents(bool update_parents);
  /**< 
   * removes the brick from (all) its parents.
   * Note that:
   * 1) Ubit objects can have several parents!
   * 2) this function is potentially dangerous: the UGroup::remove() 
   *    and UGroup::removeAll() methods should be prefered when possible.
   */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // conditionnal objects

  friend class ULink& operator/(const UCond&, UBrick&);
  friend class ULink& operator/(const UCond&, UBrick*);
  /**< 
   * conditional operator (for specifying call-back functions and conditionnal objects).
   * this operator is used for specifying call-back functions or
   * conditionnal objects (see classes UCall, UOn and UFlag). */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // errors and warnings

  virtual void error(const char *funname, const char *error) const;
  virtual void error(const char *funname, const char *error, long arg) const;
  virtual void error(const char *funname, const char *error, const char* arg) const;
  virtual void error(const char *funname, const char *error, const UStr& arg) const;
  virtual void error(const char *funname, const char *error, const std::string& arg) const;

  /**< throws an error (and prints an error message).
   * obsolete: use UError::error().
   * arguments:
   * - 'format' : same syntax as printf()
   * - 'type'   : one of 'w' (warning), 'e' (error), '!' (fatal), 'd' (debug)
   * - 'funname': name of the function where the error is issued.
   * - similar to the uerror() function */

  friend void uerror(const char *funname, const char *error);
  friend void uerror(const char *funname, const char *error, long arg);
  friend void uerror(const char *funname, const char *error, const char* arg);
  friend void uerror(const char *funname, const char *error, const UStr& arg);
  friend void uerror(const char *funname, const char *error, const std::string& arg);

  /**< throws an error (and prints an error message).
   * arguments:
   * - 'format' : same syntax as printf()
   * - 'type'   : one of 'w' (warning), 'e' (error), '!' (fatal), 'd' (debug)
   * - 'funname': name of the function where the error is issued.
   * - similar to the uerror() function */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  bool isBmode(u_modes some_Bmodes) const 
  {return ((bmodes & some_Bmodes) != 0);}
  ///< [impl] is ONE of these Brick modes verified ? (see class UMode for details).

  u_modes getBmodes() const {return bmodes;}
  ///< [impl] returns current Brick mode bitmask.

  void setBmodes(u_modes bmodes, bool on_off);
  ///< [impl] sets Brick modes to ON (resp OFF) if last arg is true (resp false). 

  void* operator new(size_t);
  void  operator delete(void*);
  ///< [impl] implementation: memory management.

protected:
  virtual void cleanup();
  /**< 
   * [impl] see <b>important note</b> on <b>subclassing</b>.
   * This function unlinks the object from its parents and destroys children 
   * (when applicable). Any class that redefines the (internal) removingFrom() 
   * function MUST HAVE A DESTRUCTOR that calls cleanup().
   */

  virtual void fire(class UEvent&, const class UOn&) const;
  ///< [impl] calls callback functions.

#ifndef NO_DOC // - - - - - - - - - - - - - - - - - - - - - - - - - - - -
public:
  friend class ULink;
  friend class UArgs;

  virtual class UProp*  propCast()   {return null;}
  virtual class UElem*  elemCast()   {return null;}
  virtual class UStr*   strCast()    {return null;}
  virtual class UGroup* groupCast()  {return null;}
  virtual class UBox*   boxCast()    {return null;}
  virtual class UWin*   winCast()    {return null;}
  ///< [impl] implementation: simulates dynamic cast.

  virtual const char* cname() const {return getClassName();}
  ///< [impl] shortcut: same as getClassName().

  virtual void addingTo(class ULink *selflink, class UGroup *parent);
  virtual void removingFrom(class ULink *selflink, class UGroup *parent);
  ///< NOTE that removingFrom() requires a destructor to be defined.

  const class ULLChain& getParentList() const {return parents;}
  ///< [impl] direct acces to object's parents

  bool hasValidParent() const;
  bool hasValidParent(bool& fake_or_deleted_par) const;

  virtual class ULink* makeLink() {return new ULink(this);}

  void addRef() {refcount++;}
  void removeRef();
  u_count getRefCount() const {return refcount;}

protected:
  class ULLChain parents;
  class UChain*  cache;  // attributes and callbacks
  u_modes  bmodes;
  u_count refcount;
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/* uref template : Ubit References (smart pointers).
 *
 * Ubit references (called "urefs") make it possible to handle objects
 * created in the heap (= in dynamic memory) in a safe way. They should
 * <b>always</b> be used for pointing objects created by 'new' or 
 * 'creator shortcuts' (such as ubutton(), ugroup(), ustr(), etc). 
 *
 * Ubit references are somewhat similar to Java references. 
 * Objects pointed by urefs are <b>automatically destroyed</b> when:
 * - they are not pointed by any uref any longer, AND
 * - they do not have parents in the GUI graph, AND
 * - they have been allocated in the heap.
 *
 * !CAUTION: 'delete' can not be used to destroy object pointed by urefs
 * (this would produce an error and the object would not be destroyed).
 * To destroy an object : set all urefs that point to it to null (and 
 * remove this object from the GUI graph).
 *
 *  Examples:
 *  <pre><tt>
 *     uref<UStr> p1 = new UStr("111");   // p1 points to a new string object
 *     uref<UStr> p2 = ustr("222");       // ustr(..) is a 'creator shortcut'
 *                                        // that calls: new UStr(...)
 *
 *     *p2 = "xxx";            // changes the CONTENT of object pointed by p2
 *     (*p2).set("xxx");       // idem
 *     p2->set("xxx");         // idem
 *     cout << *p2 << endl;    // prints the CONTENT of object pointed by p2
 *
 *     p2 = ustr("yyy");       // changes the OBJECT pointed by p2
 *
 *
 *     p1 = p2;     // p1 points to the same object as p2. the object previously
 *                  // pointed by s1 is deleted (if not pointed elsewhere)
 *
 *     p1 = null;   // s1 points to null. the object previously pointed by s1
 *                  // is NOT deleted (it is still pointed by p2)
 *
 *     p2 = null;   // s2 points to null. the object previously pointed by s2
 *                  // is deleted (if not pointed elsewhere)
 */
template <class CC>
class uref {
  CC* obj;
  CC& operator[](int i);  ///< [] meaningless and forbidden.

public:
  uref() {obj = null;}
  uref(CC& _obj) {obj = &_obj; obj->addRef();}
  uref(CC* _obj) {obj = _obj; if(obj)obj->addRef();}
  ///< constructors: this uref points to the object given as an argument (can be null).

  uref(const uref<CC>& ref2) {obj = ref2.obj; if(obj)obj->addRef();}
  ///< constructor: this uref points to the object pointed by 'ref2' (can be null).

  ~uref() {if (obj) obj->removeRef();}
  ///< destructor: deletes pointed object (if not null and not pointed elsewhere).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  uref<CC>& operator=(CC& _obj) {
    if(obj)obj->removeRef(); obj = &_obj; if(obj)obj->addRef(); return *this;
  }
  uref<CC>& operator=(CC* _obj) {
    if(obj)obj->removeRef(); obj = _obj; if(obj)obj->addRef(); return *this;
  }
  ///< assignment: this uref points to the object given as an argument (can be null).

  uref<CC>& operator=(const uref<CC>& ref2) {
    if(obj)obj->removeRef(); obj = ref2.obj; if(obj)obj->addRef(); return *this;
  }
  ///< assignment: this uref points to the object pointed by 'ref2' (can be null).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  operator CC* () const {return obj;}
  ///< type conversion: allows for:  CC* obj = ref;  if 'ref' is an uref<CC>.

  CC& operator*() const {
    if (!obj) uerror("uref::operator*", "attempt to derefence a uref (Ubit reference) that points to null [ref: %ld]", (long)this);
    return *obj;
  }
  ///< dereferencing: checks if pointed object is not null.

  CC* operator->() const {
    if (!obj) uerror("uref::operator->", "attempt to derefence a uref (Ubit reference) that points to null [ref: %ld]", (long)this);
    return obj;
  }
  ///< dereferencing: checks if pointed object is not null.

};


#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
