/* ==================================================== ======== ======= *
 *
 *  ucond.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _ucond_hh
#define	_ucond_hh
//pragma ident	"@(#)uond.hh	ubit:2.11.0"


/** base class for Ubit conditions.
 *  Notes:
 * - this class (and its subclasses: UOn, UFlag, etc.)
 *   are NOT derived from UBrick (=> they can't be UBox children)
 * - !CAUTION: NEVER delete UCond (and subclasses) instances
 */
class UCond {
public:
  virtual bool verifies(const UContext*, const class UCtrl*) const = 0;
  virtual void setParentModes(UCtrl *parent) const {};
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** event conditions.
 * for specifying conditionnal objects and callback functions
 * - general usage:  condition / object
 * - callbacks:      condition / ucall(...)
 * - see also: ucall() template, UCall class, ucall.hh header.
 * - !CAUTION: NEVER delete UOn (and subclasses) instances
 */
class UOn : public UCond {
  friend class UCtrl;
  friend class UGroup;
public:
  u_modes bmodes, cmodes;
  u_state istate;
public:
  static UOn 
    /** no action event.
     * used for style specs. 
     */
  idle,
    
    /** high-level activation event.
     * event received by activable boxes (eg. UButton, UCheckbox, UTextbox).
     * Notes:
     * - Boxes can be made activable by setting their CAN_ACTION mode.
     * - This event is discarded if the box is <b>not enabled</b> 
     *   (see: UGroup::enable())
     */
    action,

    /** arm/disarm button events.
     * events received by armable boxes (eg. UButton, UCheckbox) when 
     * they are activated (event order is: arm / action / disarm)
     * Notes: 
     * - Boxes can be made armable by setting their CAN_ARM mode
     *   (see: UMode::canArm and UGroup::setCmodes())
     * - these events are discarded if the box is <b>not enabled</b> 
     *   (see: UGroup::enable())
     */
    arm, disarm,

    /** mouse events.
     * Notes:
     * - mmove is received by the box where the mouse is currently located
     * - mdrag and mrelease are received by the box that got the mpress
     * - mrelax is similar to mrelease but it is received by the box
     *   where the mouse was released (instead of the box that got mpress)
     */
    enter, leave, 
    mpress, mlongpress, mrelease, 
    mdrag, mmove, mrelax,

    /** mouse clicks.
     * mclick is received when a mouse click is performed on a box, 
     * mbiclick when a double click is performed, etc.
     * Notes:
     * - a double click first produces a mclick then a mbiclick (and so on 
     *   for other multiple click events)
     * - mouse clicks are not emitted if the mouse has been moved more
     *   that a certain radius between the mouse press and release
     * - UOn::action is received after the mouse clicks
     */
    mclick, mbiclick, mtriclick, mquadclick,

    /** key events.
     * Notes:
     * - kpress and krelease are produced by raw key events
     * - ktype is a higher-level event that compose ASCII characters
     *   it is not activated by special or function keys (eg. LEFT, F1...)
     */
    ktype, kpress, krelease,

    /** box selection.
     * events received by selectable boxes (eg. UCheckbox) when they are
     * selected/unselected. 
     * Note: 
     * - any box can be made selectable by setting its CAN_SELECT mode
     *   (see: UMode::canSelect and UGroup::setCmodes())
     */
    select, unselect,

    /** box child selection.
     * events received by certain boxes when their (direct or indirect)
     * children are selected/unselected.
     * NOTE: these events are only received by boxes that contains
     * selections such as UListbox or UFilebox
     */
    childSelect, childUnselect,

    /** value change.
     * events received by UProp, UElem, UStr... objects when their value 
     * is changed.
     */
    change,

    /** box child change.
     * events received by boxes when the value of their UProp, UElem, UStr...
     * children is changed.
     * Note: 
     * - elemChange and strChange are both emitted by UStr children
     *   (in this order)
     */
    propChange, elemChange, strChange, 

    /** adding and removing children.
     * events received by the parent(s) of the objects that are being 
     * added, removed or deleted (deleted objects are first removed).
     */
    addChild, removeChild,

    close, deleting,

    /** drag and drop.
     */
    dragStart, dragDone, dropEnter, dropLeave, dropDone,

    /** View events.
     * These events are related to object' *views*
     * (and thus, may be produced several times for a single Box object
     *  if this object has several parents)
     */
    viewPaint, viewResize, viewMove,

    /** detects events *before* they are sent to children (see notes).
     * Note: use UBox::onChildEvent() that provides a more convenient
     * interface: this method:
     * - makes its possible tp specify event conditions
     * - detects events *after* they where received by children (this is 
     *   usually what should be done because many event types are unknown
     *   until processed by their target) */
    preChildEvent,

    /** low level native events.
     * these events are implementation dependant.
     * They are only received by "hard windows" (typically the main frame).
     * see class UWin for details on windows.
     */
    message,			///< X client message events
    anyEvent;			///< any X Event


  // Interaction State
  enum ISTATE {
    ANY = -1, 
    IDLE = 0,			// must be 0 (index in style arrays)
    DISABLED, ENTERED, ARMED, ACTIONED, DROPPED, DRAGGED,
    ACTION_COUNT			// (used in Style Arrays)
    // !! BEWARE must be coherent with typedef u_state in udefs.hh !!
  };

  UOn(u_state istate, u_modes bmodes, u_modes cmodes);
  /**< !CAUTION: NEVER delete UOn (and subclasses) instances.*/

  virtual bool verifies(const UContext*, const class UCtrl*) const;
  virtual void setParentModes(UCtrl *parent) const;
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Flagging conditions.
 * for specifying conditionnal objets that are only active when this
 * condition is verified
 * - usage:       flag / object
 * - works with:  UFlagDef
 * - !CAUTION: NEVER delete UFlag instances
 */
class UFlag : public UCond {
protected:
  const char *flagname;
public:
  static const UFlag none;

  UFlag(const char* = null);
  /**< !CAUTION: NEVER delete UFlag instances.*/
  friend UFlag& uflag(const char* name) {return *new UFlag(name);}

  const char* getName() const {return flagname;}
  virtual bool verifies(const UContext*, const class UCtrl*) const;
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Scaling conditions (for semantic zooming).
 * specify conditionnal objets that are only active when the scale 
 * condition is verified.
 * - usage:       scalerange / object
 * - works with:  UScale
 * - !CAUTION: NEVER delete UScaleRange instances
 */
class UScaleRange : public UCond {
protected:
  int scale_min, scale_max;
public:
  UScaleRange(int, int);
  ///< !CAUTION: NEVER delete UScaleRange instances.

  friend UScaleRange& uscaleRange(int scale_min, int scale_max);

  virtual bool verifies(const UContext*, const class UCtrl*) const;
};

/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
#endif

