/* ==================================================== ======== ======= *
 *
 *  udisp.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _udisp_hh
#define	_udisp_hh
//pragma ident	"@(#)udisp.hh	ubit:2.11.0"

/** Logical Display Context.
 * this class modelizes the physical X Display.
 * it is the base class of UAppli, the Application Context.
 */
class UDisp {
public:
  UDisp(class UAppli&, const char* display_name = null);
  /**<
   * creates a new Logical Display.
   * display_name is the name of the X server (eg. "computername:0")
   * default = null (the current X server) */

  virtual ~UDisp();

  int setTrueColors(int depth_hint);
  int setPseudoColors(int depth_hint);
  /**< 
   * attempts to set TrueColor or PseudoColor mode with this depth.
   * Notes:
   * - default is server dependant
   * - TrueColor is necessary if you use alpha blending (transparency)
   * - tries to find the closest compatible depth if 'depth_hint' is 
   *   not supported
   * - returns the actual depth that was set. 0 means that no compatible
   *   depth could be found for this mode
   * - must be called before realize() or mainLoop() */

  bool setScreen(int screen_no);
  /**<
   * changes screen according to number (default is screen #0).
   * returns false if screen number is invalid.
   * must be called before realize() or mainLoop() */

  virtual void realize();
  /**<
   * realizes this display (allocates the X resources).
   * Note: this function must:
   * - always be called <b>once</b>
   * - be called <b>after</b> setPseudoColorMode(), setTrueColorMode()
   *   or setScreen(), (if you call these functions) */

  virtual bool isRealized() const;
  ///< is this Display initialized?.

  int getScreenWidth() const;
  int getScreenHeight() const;
  ///< size of the Screen used by this UDisp.

  int getDepth() const;
  ///< depth of this specific UDisp.

  int getScreenDefaultDepth() const;
  /**< default depth of the Screen used by this UDisp.
   * Note: this value may differ from getDepth() */

  int getScreenNumber() const;
  ///< number of the Screen used by this UDisp.

  int getScreenCount() const;
  ///< total number of Screens available on this X Server.

  const char* getDisplayName() {return display_name;}
  ///< name of the X server (default = null).

  class UAppli& getAppli() const {return *appli;}
  ///< the corresponding Application Context.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  class UNatDisp* getNatDisp() const {return natdisp;}
  ///< [impl] implementation dependant data.

  // will manage the events of this window if it is realized and shown
  void addHardwin(class UWin*);
  void removeHardwin(class UWin*);

  // returns true if OK; use default otherwise
  bool realizeFont(const class UFont&);
  bool realizeColor(class UColor&);
  bool realizeCursor(class UCursor&);

#ifndef NO_DOC
  class UMSclient* umsclient;

protected:
  friend class UConf;
  friend class UWin;
  friend class UGraph;
  friend class UWinGraph;
  friend class UNatDisp;

  static int last_id;
  const int id;
  class UAppli* appli;       // Corresponding UAppli
  char* display_name;        // name of the X Display (null = default)
  class UNatDisp* natdisp;   // Native Display
  UChain hardwin_list;	     // list of hardwins that are event managed

  UDisp();                   // used by UAppli
#endif
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
