/* ==================================================== ======== ======= *
 *
 *  uevent.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uevent_hh
#define	_uevent_hh
//pragma ident	"@(#)uevent.hh	ubit:2.11.0"


/** X Event pointer (hidden type).
*/
typedef union _XEvent* UX_Event; 

/* ==================================================== ======== ======= */

/** the Properties of the Source and the Source View.
*/
struct USourceProps {
  bool  enabled;
  ///< is the source enabled ?.

  bool  autoClose;
  ///< is the source in auto_close mode ?.

  bool  redrawStatus;
  URegion redrawClip;
  ///< redrawClip is the region that must be redrawn (defined only if redrawStatus is true).

  class UView*   opaqueView;
  ///< the last opaque view beneath the sourceView.

  class UView*   layoutView;
  ///< layout should take place from this view.

  const UCursor* cursor;
  ///< the cursor of the sourceView.

  class UGroup   *browseGroup, *parentBrowseGroup;  ///< [impl].
  class UContext *parentContext;  ///< [impl].

  USourceProps(UView* hardwin_view) {reset(hardwin_view);}

  void reset(UView* hardwin_view);
  void set(UView* view, UGroup*, const UContext& curp);
};

/* ==================================================== ======== ======= */

/** Ubit Event class.
 */
class UEvent {
public:
  /// standard Event IDs (correspond to UOn conditions: see class UOn for details).
  enum {
    anyEvent  	= 0,	  // for UOn::anyEvent
    mpress	= 1<<0, 
    mrelease	= 1<<1, 
    mmove 	= 1<<2, 
    mdrag 	= 1<<3, 
    kpress 	= 1<<4, 
    krelease 	= 1<<5, 
    enter 	= 1<<6, 
    leave 	= 1<<7, 
    change 	= 1<<8,
    select 	= 1<<9,
    action      = 1<<10,
    add		= 1<<15,
    remove	= 1<<16,
    close       = 1<<17,
    deleting    = 1<<18,
    viewPaint 	= 1<<20, 
    viewResize 	= 1<<21, 
    viewMove 	= 1<<22, 
    search	= 1<<23, 
    input	= 1<<24, 
    timeout	= 1<<25, 
    message 	= 1<<26  //!BEWARE: no comma after last item!
  };

  /// mouse button IDs for UEvent::getButtons().
  static const int MButton1, MButton2, MButton3, MButton4, MButton5;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // source objects and source view

  class UGroup* getSource() const;
  ///< returns the UBox or the UGroup that produced this event.

  class UView* getView() const {return sourceView;}
  ///< returns the source VIEW that received this event (note that a box can have several Views)

  class UProp* getPropSource() const;
  class UElem* getElemSource() const;
  class UStr*  getStrSource() const;
  ///< (for UOn::change callbacks) returns the UBrick that produced this event.

  class UBrick* getTarget() const;
  ///< (for UOn::childSelect/add/remove callbacks) returns the child.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // properties

  int getID() const {return id;}
  ///< returns the event id (see: UEvent::ID).

  const class UCond* getCond() const {return cond;}
  /**< returns the xxx condition of the:  UOn::xxx / ucall( ).
   * Note: as the same UEvent can be catched by several ucall() conditions
   * this value should be checked immediately in the callback function
   * as it can change at a later time */

  class USourceProps& getSourceProps() {return sp;}
  ///< gets the source properties.

  u_time getTime() const;
  ///< returns the time when this event was produced (when available).

  class UInputFlow* getFlow() const {return iflow;}
  int getFlowID() const;
  ///< returns the Input Flow that produced this event.

  class UAppli* getAppli() const;
  ///< returns the Application Context where this event was produced.

  class UWin*  getHardWin() const;
  class UView* getHardWinView() const;
  ///< returns the HARD Window that received this event.

  UX_Event getXEvent() const {return xev;}
  ///< implementation dependent : returns the X Event (NB: only valid at reception time).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // mouse location

  u_pos getX() const;
  u_pos getY() const;
  /**< (for mouse and key events) return the location of the mouse in the Source VIEW .
   * Returns -1 if source view wasn't found (= when getView() returns null) */

  u_pos getXscreen() const;
  u_pos getYscreen() const;
  ///< (for mouse and key events) return the location of the mouse on the SCREEN.

  u_pos getXwin()  const {return xmouse;}
  u_pos getYwin()  const {return ymouse;}
  /**< return the location of the mouse in the WINDOW that caught this event.
   *   this location is relative to the window returned by getWin())
   */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // mouse buttons

  int getButtons() const;
  /**< (mouse events) returns the button(s) that are currently being pressed.
   * details:
   * - for press and release events:
   *   value is ONE of UEvent::MButton1, UEvent::MButton2, etc.
   * - for move or drag events:
   *   value is an ORed COMBINATION of these values (several buutons can be
   *   pressed simultaneously).
   * - returns 0 if no button is pressed.
   */

  int getClickCount() const;
  ///< (mouse events) returns the number of mouse clicks. 

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // keyboard

  int getKeyChar() const;
  ///< (key events) returns the typed character (in ASCII encoding).

  unsigned long getKeySym() const;
  ///< (key events) returns the X KeySym (NB: key modifiers (Shift, Alt..) are not taken into into account).

  int getMods() const;
  ///< (mouse and key events) returns a combination of key and mouse button modifiers.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // element retrieval

  class UElem* getElem();
  ///< returns the element that was under the mouse (null if none found).

  class UStr*  getStr();
  /**< returns the UStr that was under the mouse (null if no str found).
   * Note: same function as getElem() but discards elements that 
   * are not strings.*/

  class UElem* getElem(class UElemProps&);
  class UStr*  getStr(class UElemProps&);
  /**<
   * variants of getElem() and getStr() that return useful information.
   * argument UElemProps (see this class) is filled with useful data such
   * as the elem location, its graphical properties, etc.
   *
   * Note on getStr() : the region returned in UElemProps does not contain
   * the entire string but just the line that is beneath the mouse 
   * (in case of UFlowView rendering) */

  class UElem* searchElem(class UElemProps&, const UElem* searched_elem,
			  int strpos1 = -1, int strpos2 = -1);
  /**<
   * searches the element that is given as an argument.
   * returns useful information about this element in UElemProps
   * notes:
   * - args. strpos1, strpos2 are used when searching for UStr(ings)
   * - the region returned in UElemProps does not contain the entire string
   *   but just its 'strpos1' to 'strpos2' subpart */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // event management for transparent tools

  void goThrough(const class UBox*);
  ///< events will go through this object.

  void setMouse(u_pos x, u_pos y);
  ///< changes mouse coordinates (x,y are in window coordinates).

  void addFlag(const class UFlag&);
  ///< adds a Flag to the Event (for transparent tools, etc); see: UFlag. 

  class UFlag** getFlags()   const {return flags;}
  class UBrick** getExts()   const {return exts;}
  u_card getFlagCount()      const {return flagCount;}
  u_card getExtCount()       const {return extCount;}
  /**< 
   * retreive Flags and Extensions (in callback functions).
   * Note: the returned data is automatically destroyed when the UEvent
   * is destroyed. NEVER delete it explicitely nor use it after
   * the UEvent is deleted. */

  // Queries the mouse location and initializes the UEvent fields.
  // Returns true if the mouse is located on a UWin of the UAppli
  // Example:
  //     UEvent e;
  //     if (e.queryMouse()) {      // mouse over an appli window
  //        box = e.getSource(); view = e.getView(); 
  //        x_in_view = e.getX();
  //        x_in_win = e.getXwin();
  //     }
  //     e.whereInScreen(&x_in_screen, &y_in_sceen);
  //
  //NOT_AVAILABLE: bool queryMouse();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  UEvent(int event_id, class UInputFlow*, class UView* hardwin_view, UX_Event);
  ///< [impl] 'win_view' must be a valid UWin view or null.

  ~UEvent(); 

#ifndef NO_DOC
private:
  friend class UInputFlow;
  friend class UCtrl;
  friend class UBox;
  friend class UView;
  friend class UGraph;
  friend class UNatWin;

  class UGroup*  source;	 // the box or group that produced this event
  class UView*   sourceView;     // the view that contains the mouse
  class UView*   hardWinView;    // the view of the containing HARD window
  class UBrick*  aux;            // target or non group source
  class UInputFlow* iflow;       // input flow that produced this event
  USourceProps   sp;		 // Source properties 
  const class UCond* cond;	 // UOn condition of the ucall()
  const class UBox* through;     // events go through this transparent tool
  class UFlag**  flags;		 // event flags
  class UBrick** exts;	         // extensions
  unsigned short flagCount, extCount;
  bool   propagate;              // propagate the event
  int    id;	                 // event id
  int    detail;                 // additional data
  u_time time;	                 // event time stamp
  u_pos  xmouse, ymouse;         // current mouse position in the HARD window
  u_pos  xdrag_ref, ydrag_ref;   // screen coords of the View that got the mpress
  UX_Event xev;	                 // pointer to X event (hidden type)

public:
  void setID(int);               // sets Event ID
  void setSource(UView*);        // sets the View & the Box that produced this event
  void setSource(UGroup*);       // sets the Group (when no View is available)
  void setAux(UBrick*);
  void setCond(const UCond*);
  void setTime(u_time t)    {time = t;}
  void setPropagate(bool s) {propagate = s;}
  bool getPropagate()       {return propagate;}

  UView *locateSource(UView *searched_view);
  UView *locateSource(u_pos x_in_hardwin, u_pos y_in_hardwin);
  ///< [impl] returns UView if found and null otherwise.

  void actualize(int event_id, u_pos x_in_hardwin, u_pos y_in_hardwin, UX_Event ev)
  {id = event_id; xev = ev; xmouse = x_in_hardwin; ymouse = y_in_hardwin;}
  ///< [impl] actualize events with new Event ID and new coordinates.

  void copy(const UEvent&);
  ///< [impl] copies UEvent content AND the content of all pointed fields.
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Useful information on UElem objects that are retrieved 
 *  by UEvent::getElem().
 */
class UElemProps {
public:
  class UElem* elem;
  class ULink* elemLink;
  /**< points to the element that is located under the mouse when calling 
   *  UEvent::getElem()/getStr(). */

  URegion region;
  /**< the region that encloses this element. 
   * NOTE that these coordinates are relative to the enclosing WINDOW 
   * (not the elem parent!). use methods getX(), getY() for obtaining 
   * local coordinates. */

  bool exactMatch;
  /**< true if this element is exactly located under the Mouse.
   * false if this elem is the last element before the Mouse position */

  int strpos, strpos2;
  ///< strpos = position of the mouse in the string if elem is an UStr.

  class UWinContext* winContext;
  class UContext*    elemContext;
  /**< graphical contexts of the enclosing window and this element.
   * Note: this data is automatically deleted when UElemProps is deleted */

public:
  UElemProps(); 
  ~UElemProps();

  u_pos getX(class UEvent*);
  u_pos getY(class UEvent*);
  /**< return the location of the element relatively to the source VIEW.
   * (ie. the view returned by e.getView()) */

  u_dim getWidth();
  u_dim getHeight();
  ///< return the width and the height of the element.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

#ifndef NO_DOC
  friend class UEvent;
  friend class UView;

  void set(UContext&, UElem*, class ULink*, const URegion&, 
	   int _strpos, bool exact_match);
  void merge(UContext&, UElem*, class ULink*, const URegion&,
	     bool exact_match);
#endif
};


#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */

