/* ==================================================== ======== ======= *
 *
 *  ugadgets.hh
 *  Ubit Project [Elc::2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _ugadgets_hh
#define	_ugadgets_hh
//pragma ident	"@(#)ugadgets.hh	ubit:2.11.0"
#include <ustr.hh>


/** Separator gadget
 */
class USepar: public UBox {
public:
  static UStyle *hstyle, *vstyle;

  USepar(const UArgs& a = UArgs::none);
  friend USepar& usepar(const UArgs& a = UArgs::none) {return *new USepar(a);}

  virtual const UStyle& getStyle(const UBox*) const;
  static  const UStyle& makeVStyle();
  static  const UStyle& makeHStyle();
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Label gadget.
 *  Geometry:
 *  - horizontal lay out by default (add: UOrient::vertical otherwise). 
 *  - this object is resized when its children are changed (or resized).
 *  - consider UTextbox an subclasses for displaying textual messages
 *    that are dynamically changed.
 *
 *  Default Properties: 
 *  - font and foreground color are inherited from parents.
 *  - background is transparent (= UBgcolor::none)
 *
 *  Example:
 *  <br>  ULabel& my_label = ulabel( uima("whatever.jpg")+ " My Label" )
 *  <p>Note: 
 *  <br>  " My Label"  is equivalent to:  ustr(" My Label")
 */
class ULabel: public UBox {
public:
  static UStyle *style;

  ULabel(const UArgs& a = UArgs::none);
  ///< constructor; see also ~UGroup() and the <em>creator shortcut</em> <b>ulabel()</b>

  friend ULabel& ulabel(const UArgs& a = UArgs::none) {return *new ULabel(a);}
  ///< <em>creator shortcut</em> that is equivalent to *new ULabel().

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static  const UStyle& makeStyle();
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Active Item gadget.
 *  Active gadget that behaves as a UButton but has no decoration. 
 *  It is typically used as a child of a UListbox or of a UTrow in a UTable 
 *  (when active items a prefered to UTcells).
 *
 *  Geometry: same as ULabel
 *
 *  Default Properties: same as ULabel when object is idle.
 */
class UItem: public UBox {
public:
  static UStyle *style;

  UItem(const UArgs& = UArgs::none);
  friend UItem& uitem(const UArgs& a = UArgs::none) {return *new UItem(a);}

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static  const UStyle& makeStyle();
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Button gadget.
 *  Similar to the UItem gadget but has decorations. These decorations are
 *  determined in a dynamical way depending on the button's parents.
 *
 *  Geometry: same as ULabel
 *
 *  Default Properties: 
 *  - font and foreground color are inherited from parents.
 *  - background color depends on parent. It is transparent if the button
 *    is idle and inside a UMenu, a UMenubar or a UBar, and grey otherwise.
 *
 *  <p>Example:
 *  <br> UButton& save_button = 
 *  <br>   ubutton( UPix::diskette + " Save..."  + ucall(this, &MyClass::save) )
 *  <p>With: 
 *  <p>    void MyClass::save(UEvent& e) { ... }
 *  <p>Note: 
 *  <br>   " Save..."   is equivalent to:   ustr(" Save...")
 */
class UButton: public UBox {
public:
  static UStyle *style, *menuStyle, *barStyle, *flatStyle, *linkStyle;
  ///< contextual styles depending on the Button's parent

  UButton(const UArgs& a = UArgs::none);
  ///< constructor; see also ~UGroup() and the <em>creator shortcut</em> <b>ubutton()</b>

  friend UButton& ubutton(const UArgs& a = UArgs::none) {return *new UButton(a);}
  ///< <em>creator shortcut</em> that is equivalent to *new UButton().

  virtual const UStyle& getStyle(const UBox* box_parent) const;
  /**<
   * returns the contextual UStyle of this object.
   *  This virtual function calls one of the makeXxxStyle() functions that
   *  are redefined for this specific class.
   * - see also: UStyle and makeXxxStyle() functions */

  static const UStyle& makeStyle();
  static const UStyle& makeMenuStyle();
  static const UStyle& makeBarStyle();
  static const UStyle& makeFlatStyle();
  static const UStyle& makeLinkStyle();
};

/* ==================================================== ======== ======= */

/** Flat Button gadget.
 * has a "flat" square border
 */
class UFlatbutton: public UButton {
public:
  UFlatbutton(const UArgs& a = UArgs::none);

  friend UFlatbutton& uflatbutton(const UArgs& a = UArgs::none) 
  {return *new UFlatbutton(a);}

  virtual const UStyle& getStyle(const UBox*) const {return UButton::makeFlatStyle();}
};

/* ==================================================== ======== ======= */

/** Link Button gadget.
 * resembles an HTML link (no border and specific properties)
 */
class ULinkbutton: public UButton {
public:
  ULinkbutton(const UArgs& a = UArgs::none);

  friend ULinkbutton& ulinkbutton(const UArgs& a = UArgs::none) 
  {return *new ULinkbutton(a);}

  virtual const UStyle& getStyle(const UBox*) const {return UButton::makeLinkStyle();}
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Radio Button gadget.
 */
class URadiobutton: public UButton {
public:
  URadiobutton(const UArgs& a = UArgs::none);

  friend URadiobutton& uradiobutton(const UArgs& a = UArgs::none) 
  {return *new URadiobutton(a);}

  virtual const UStyle& getStyle(const UBox* box_parent) const;
};

/** Checkbox gadget
 */
class UCheckbox: public UBox {
public:
  /// three contextual styles depending on the Button's parent
  static UStyle *style, *menuStyle, *barStyle;

  UCheckbox(const UArgs& = UArgs::none);
  friend UCheckbox& ucheckbox(const UArgs& a = UArgs::none) {return *new UCheckbox(a);};

  virtual const UStyle& getStyle(const UBox* box_parent) const;
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Textbox gadget.
 *  This gadget should be used for displaying (or editing) text.
 *
 *  Geometry:
 *  - horizontal lay out by default (add: UOrient::vertical otherwise). 
 *  - this object is NOT resized when its children are modified (ie. added,
 *    removed or resized) (= this object will keep its initial width).
 *  - The width can be specified by adding an UWidth child. It can also
 *    be changed through user interaction if this object is included
 *    in a box that is "flexible" in the horizontal direction (see UBox).
 *  - UTextbox (and UFlowbox) should be used for displaying textual messages 
 *    that are dynamically changed for optimization purpose.
 *
 *  Default Properties: 
 *  - font and foreground color are inherited from parents.
 *  - background is transparent (= UBgcolor::none)
 *
 *  Example 1: non editable text:
 *  <br>  UStr& my_mesg = ustr("Can't open file");
 *  <br>  utextbox( UPix::ray + "Warning: Error: " + my_mesg )
 *  <p>Then:
 *  <br>  my_mesg = "Can't do it";       // changes the message in the Textbox
 *
 *  Example 2: editable text:
 *  <br>  UStr& my_text = ustr("");      // uedit() makes text editable
 *  <br>  utextbox( uedit() + my_text + UPix::question ) 
 * 
 *  Example 3: editable text with callback:
 *  <br>  utextbox( uedit() + my_text + ucall(this, &MyClass::getEntry, &my_text) )
 *  <br>With: 
 *  <br>  void MyClass::getEntry(UStr* s)  {cout << s->chars() << endl;}
 */
class UTextbox: public UBox {
public:
  static UStyle *style;

  UTextbox(const UArgs& = UArgs::none);
  ///< constructor; see also ~UGroup() and the <em>creator shortcut</em> <b>utextbox()</b>

  friend UTextbox& utextbox(const UArgs& a = UArgs::none) {return *new UTextbox(a);}
  ///< <em>creator shortcut</em> that is equivalent to *new UTextbox().

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static  const UStyle& makeStyle();
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** List gadget.
 * Any gagget or UBox can be used as a child but UItem (an active gadget
 * that behaves as a Button but have no decoration) is generally prefered.
 *
 *  Geometry:
 *  - vertical lay out by default (add: UOrient::horizontal otherwise).
 *
 *  Default Properties: 
 *  - font and foreground color are inherited from parents.
 *  - background is white (= UBgcolor::none)
 */
class UListbox: public UBox {
public:
  static UStyle *style;

  UListbox(const UArgs& a = UArgs::none);
  ///< class to be completed...

  friend UListbox& ulistbox(const UArgs& a = UArgs::none) {return *new UListbox(a);}

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static  const UStyle& makeStyle();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  // implementation
  virtual UGroup* getBrowseGroup();
protected:
  UCall &mouse_call;
  virtual void mouseCB(UEvent&);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Spinbox gadget.
 */
class USpinbox : public UBox {
public:
  static UStyle *style;

  USpinbox(const UArgs& = UArgs::none);
  ///< standard constructor.

  USpinbox(UIntg& value, const UArgs& = UArgs::none);
  USpinbox(UIntg& value, UIntg& increment, const UArgs& = UArgs::none);
  /**< constructors for specifing a shared 'value' or 'increment'.
   * 'value' and 'increment':
   * - are NOT duplicated internally and must NOT be destroyed by 'delete'
   *   (nor be allocated in the stack).
   * - can be shared by other objects (their value can be changed by clients)
   * - are automatically deleted (when the Spinbox is deleted and no uref 
   *   points to them).
   */

  friend USpinbox& uspinbox(const UArgs& a = UArgs::none) 
  {return *new USpinbox(a);}
  ///< <em>creator shortcut</em> that is equivalent to *new USpinbox().

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  static  const UStyle& makeStyle();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void setValue(const UIntg&);
  virtual void setIncrement(const UIntg&);

  virtual const UIntg& getValue() const;
  virtual const UIntg& getIncrement() const;

  virtual UIntg& value() const     {return *pvalue;}
  virtual UIntg& increment() const {return *pincrement;}

private:
  uref<UIntg> pvalue, pincrement;
  UStr str;
  UTextbox textbox;

  virtual void constructs(const UArgs& a);
  virtual void syncVals(int dir);
  virtual void doAction();
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
