/* ==================================================== ======== ======= *
 *
 *  ugraph.hh
 *  Ubit Project [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _ugraph_hh
#define	_ugraph_hh
#include <uappli.hh>
//pragma ident	"@(#)ugraph.hh	ubit:2.10.1"

/** client class for drawing graphics.
 * note: do not use the UWinGraph class that is reserved for the Ubit lib)
 */
class UGraph {
  friend class UGraphElem; 
protected:
  class UView* boxview;      // logical View where the graphics are drawn
  class UWinGraph* wg;       // Window Graphics context
  class UNatGraph* natgraph; // (hidden) Native Graphic Context
  class UNatDisp*  natdisp;  // (hidden) Native Display Context
  struct UGraphElem* gelem;  // graphics stack
  bool  uniform_alpha;
  float alpha;
public:
  UGraph();
  UGraph(class UView*);
  UGraph(class UEvent*);
  UGraph(class UEvent&);
  /**< 
   * creates a new UGraph for drawing in the UView given as an explicit
   * or implicit argument.
   * Function:
   * -  UGraph(UView*) allocates physical resources for drawing in this View
   * -  UGraph(UEvent*) allocates resources for drawing in the source View
   *    that is relative to this Event (ie. UEvent::getView()) and sets
   *    the clipping area according to UEvent::redrawClip when applicable.
   *    This makes it possible to redraw just the subpart of the View
   *    that needs to be refreshed.
   * 
   * Typical usage:
   * <br> void callback_function(UEvent* e) {
   * <br>     UGraph g(e);
   * <br>     g.drawLine(...);
   * <br> }
   *
   * Notes:
   * - as UGraph objects allocates physical resources, they should be
   *   created in the stack (as in the previous example) in order to
   *   be released as soon as possible. Creating several UGraphs by using
   *   the 'new' primitive is not a good idea and can pose problem
   *   with certain implementations.
   *
   * - The Font and the foreground and background Colors are undefined 
   *   when the UGraph is created and must be set explicitely (when needed)
   *
   * - The default drawing mode (called 'PaintMode') overwrites pixels.
   *   method setXORMode() sets the 'XORMode' that alternates the current
   *   foreground and background Colors.
   */
  virtual ~UGraph();

  // ==== Related objects ==================================================

  UView *getView() const {return boxview;}
  /**< returns the current View.
   *  the 'current view' is the UView where the graphics are being drawn */

  UDisp* getDisp() const;
  /**< returns the logical Display of this View.*/

  UAppli* getAppli() const;
  /**< returns the Application context this View belongs to.*/

  // ==== Modes and Attributes =============================================

  void setColor(const UColor* c);
  void setColor(const UColor& c) {setColor(&c);}
  /**< set the Foreground color.
   * note: the color is undefined when the UGraph is created */

  void setBgcolor(const UColor* c);
  void setBgcolor(const UColor& c) {setBgcolor(&c);}
  /**< set the Background color.
   * note: the color is undefined when the UGraph is created */

  void setFont(const UFont*);
  void setFont(const class UFontDesc*);
  /**< set the current Font.
   * notes:
   * - the font is undefined when the UGraph is created
   * - UFontDesc is an internal representation of UFont (see file ufont.hh) */

  void setXCursor(const UCursor*);
  /**< changes the mouse Cursor.*/

  bool isXORMode();
  void setXORMode();
  void setPaintMode();
  /** these functions set the drawing mode. 
   * notes:
   * - PaintMode (the default) overwrites pixels with the foreground Color
   * - XORMode alternates the current foreground and background Colors */

  void setThickness(int = 0);
  int  getThickness();
  /**< changes line thickness when drawing lines, etc.
   *  the default value 0 sets thickness to 1 pixel and optimizes drawing */

  // ==== Clip and Flush ==================================================

  void setClip(const URegion&);
  void setClip(u_pos x, u_pos y, u_dim width, u_dim height);
  /**< set the clipping zone relatively to VIEW coordinates.
   * coordinates are relative to the origin of the VIEW */

  void setWinClip(const URegion&);
  void setWinClip(u_pos x, u_pos y, u_dim width, u_dim height);
  /**< set the clipping zone relatively to WINDOW coordinates.
   * coordinates are relative to the origin of the WINDOW */

  void flush();
  /**< this function flushes drawing requests (when applicable) */

  int grabPointer(class UCursor* = null, bool confine_to_window = false); 
  /**< starts a "physical grab" on the X server.
   * warning: this function may lock the X server! */

  void ungrabPointer();
  /**< ends a "physical grab" on the X server. */

  // ==== Coordinates ======================================================

  u_pos XToXwin(u_pos x_in_view);
  u_pos YToYwin(u_pos y_in_view);
  /**< conversion from local View coords. to global Window coords.*/

  u_pos XwinToX(u_pos x_in_win);
  u_pos YwintoY(u_pos y_in_win);
  /**< conversion from global Window coords. to local View coords.*/

  // ==== Clearing and Copying routines ====================================

  void clearRect(u_pos x, u_pos y, u_dim width, u_dim height,
		 bool repaint_view = false);
  /**< clears a rectangle.
   * - if 'repaint_view' is true : repaints the normal content of the view
   *   in the specified region
   * - otherwise: fills the area with the UGraph's background Color
   * - see also: UView::updatePaint() in uview.hh for more options
   * - Note: this function is not altered by the XOR mode */

  void clearView(bool repaint_view = false);
  /**< same as clearRect() but clears the entire view.*/

  void copyArea(u_pos x, u_pos y, u_dim width, u_dim height,
		u_dim delta_x, u_dim delta_y,
		bool generate_refresh_events_when_obscured);
  /** copies a rectangular area.
   * copies a rectangular area with size (width,height) at location (x,y) 
   * to a location shifted by delta_x, delta_y
   * - note: CopyArea can't copy obscured parts of the component
   *    but will generate refresh events form repainting the missing parts
   *    if the last arg. is true */

  // ==== Drawing routines =================================================
  // Note: these functions only use the Foreground Color in PaintMode
  //       they also use the Background Color in XORMode

  // --> SEE ALSO the Pixmap and Image sections below !

  void drawArc(u_pos x, u_pos y, u_dim width, u_dim height, 
	       float from_angle, float arc_angle);
  /**< draws an arc of a circle (or a complete circle).
   * notes:
   * - x,y,width,height : top/left position and size of the enclosing rectangle
   * - angles are in degrees */

  void drawLine(u_pos x1, u_pos y1, u_pos x2, u_pos y2);
  void drawPoint(u_pos x, u_pos y);

  void drawPolyline(u_pos xpoints[], u_pos ypoints[], u_card card);
  /**< drawPolyline draws a sequence of connected lines.*/

  void drawPolygon(u_pos xpoints[], u_pos ypoints[], u_card card);  
  /**< drawPolygon draws a polygon (which is automatically closed).*/

  void drawRect(u_pos x, u_pos y, u_dim width, u_dim height);

  void drawString(const UStr *s, u_pos x, u_pos y);
  void drawString(const char *str, int str_length, u_pos x, u_pos y);
  /**< note: these functions use the current Font (see setFont()).*/

  // ==== Filling routines =================================================
  // Note: these functions only use the Foreground Color in PaintMode
  //       they also use the Background Color in XORMode

  void fillArc(u_pos x, u_pos y, u_dim width, u_dim height,
	       float from_angle, float arc_angle);
  /**< fills an arc of a circle (or a complete circle).
   * notes:
   * - x,y,width,height : top/left position and size of the enclosing rectangle
   * - angles are in degrees */

  void fillPolygon(u_pos xpoints[], u_pos ypoints[], u_card card, int mode=0);
  /**< fills a polygon (which is automatically closed).
   * argument 'mode' can be set for optimisation purpose: 
   * - by default, mode=0 and the polygon can have an arbitrary shape
   * - mode=2 assumes a Convex polygon. 
   * - mode=1 assumes a non Convex but non Intersecting polygon. */

  void fillRect(u_pos x, u_pos y, u_dim width, u_dim height);
  /**< fills the area.*/

  void fillView(); 
  /**< fills the whole View.*/

  // ==== Images and Pixmaps =============================================

  void drawIma(const class UIma*, u_pos x, u_pos y);
  void drawIma(const class UNatIma*, u_pos x, u_pos y);
  void drawIma(const class UNatPix*, u_pos x, u_pos y);
  /**<nb: the following method can aslo be used for UPix objects. */

  static void getImaSize(class UNatIma*, u_dim &w, u_dim &h);
  static void getPixSize(class UNatPix*, u_dim &w, u_dim &h);

  // ==== Font Metrics ====================================================

  int  getTextHeight(const class UFontDesc*);
  int  getTextWidth(const class UFontDesc*, const char *str, int len);
  void getTextSize(const class UFontDesc*, const char *str, int len, 
		   u_dim *width, u_dim *height);
  ///< returns font metrics.

  void getSubTextSize(const class UFontDesc*, const char *str, int len, 
		      u_dim *w, u_dim *h, u_dim available_width, 
		      int *sublen, int *change_line);
  int getSubTextSize(const UFontDesc *font, const char *s, int len, 
		     u_dim maxwidth, u_dim chw, u_dim *w, u_dim *h);
  ///< returns the length and size of the enclosed substring.
  // chercher substring jusqua:
  // - premier \n
  // - premier " " ou equivalent avant available_width
  // - a defaut renvoie tout
  // change_line : 1 si line must be changed because of size
  //   and 2 if true \n char found

  int getCharPos(const class UFontDesc*, const char *s, int len, u_pos x);
  int getXPos(const class UFontDesc*, const char *s, int len, int charpos);
  ///< returns the char pos in the string and vice-versa.

  // === Misc. ===

  class UNatWin* createBuffer(u_dim width, u_dim height);
  ///< creates an off-screen pixmap.

  void deleteBuffer(class UNatWin* altbuf);
  /**< deletes an off-screen pixmap.
   * note: argument must be a variable and is always set to null */

  class UNatGraph* getNatGraph() const {return natgraph;}
  class UWinGraph* getWinGraph() const {return wg;}
  class UNatDisp*  getNatDisp()  const {return natdisp;}
  class UNatWin*   getNatWin()   const;
  /**< [Acces to implementation-dependent hidden objects.
   * Note: these "Native Objects" encapsulate platform-dependant data.
   * their implementation is subject to change and to vary depending on
   * the native platform (=> avoid accessing these objects directly!) */

protected:
  enum GType {NONE, WIN, CLIENT, SUBWIN, DOUBLEBUF, BLEND};

  void constructs(class UView*);
  void push(GType, class UNatWin* dest, const URegion &clip, 
	    u_pos _xwin, u_pos _ywin);
  void push(GType, bool alloc, const URegion &r);
  void pop();
  void popAll();

  UNatWin* allocBuffer(u_dim width, u_dim height);
  void releaseBuffer(UNatWin *buf);
};

/* ==================================================== [Elc:01] ======= */
/* ==================================================== ======== ======= */

/** internal Graphics context for drawing Ubit objects.
 * Notes:
 * - clients should use the UGraph (<b>not</b> UWinGraph that is reserved for 
 *   the Ubit lib)
 * - UWinGraph is an optimized version of UGraph for drawing directly
 *   in Ubit windows. Its methods use <b>absolute</b> Window coordinate
 *   instead of relative coordinates in the current View
 * - note that UWinGraph does not use the same internal X GCs than UGraph
 */
class UWinGraph : public UGraph {
  friend class UGraph;
  friend class UGraphElem;
  class UDisp*   disp;	     // logical Display
  class UWin*    hardwin;    // the corresponding hard window
  class UNatWin* natwin;     // pointer to the (hidden) Native Window
public:
  UWinGraph(class UWin*);
  virtual ~UWinGraph();

  bool init(UDisp*, UView* window_view);
  /**< inits the WinGraph.
   * Note: init() does not create the X Window (this is done by UWin::realize) */

  bool isWinRealized() const;
  /**< is the X Window created ?. */

  class UDisp*  getDisp()  const {return disp;}
  class UAppli* getAppli() const {return &disp->getAppli();}

  class UWin*   getHardwin() const {return hardwin;}
  /**< returns the "hard window" that contains the current View. */

  class UNatWin*  getNatWin()  const {return natwin;}
  class UNatDisp* getNatDisp() const {return disp->getNatDisp();}

  void begin(const URegion& clip);
  /**< must be called before drawing (to allocate graphical resources).
   * the clip specificlip specifies where drawing will take place in the WINDOW.
   * a drawing sequence must start by begin() and finish by end() */

  void end();
  /**< must be called after drawing (to release graphical resources).
   * a drawing sequence must start by begin() and finish by end() */

  void beginDoublebuf(const URegion& clip);
  void beginBlend(const URegion& clip, float alpha);
  void beginSubwin(const URegion& clip, u_pos xwin, u_pos ywin);
  /**< variants of the begin() function.
   * a drawing sequence must always be terminated by end() */

  void blendColor(const URegion& clip, float alpha);
  /**< blends the 'clip' zone with current color.
   * 'alpha' is the value for alpha blending. the current color must
   * have been specified by calling setColor() */

  void setClip(const URegion&);
  void setClip(u_pos x, u_pos y, u_dim width, u_dim height);
  void setWinClip(const URegion &r);
  void setWinClip(u_pos x, u_pos y, u_dim width, u_dim height);
  /**< setClip and setWinClip are similar for this class:.
   * notes:
   * - coordinates are relative to the origin of the WINDOW in all cases
   * - the view is no taken into account
   * - see also: UGraph::setClip() and UGraph::setWinClip()
   */
protected:
  void blendLayers(UGraphElem* layer1, UGraphElem* layer2);
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
