/* ==================================================== ======== ======= *
 *
 *  ugroup.hh : Logical "grouping" container.
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _ugroup_hh
#define	_ugroup_hh
//pragma ident	"@(#)ugroup.hh	ubit:2.11.0"
#include <uargs.hh>
#include <uctrl.hh>


class UListPos {
  friend class UGroup;
  class ULink* link;
  int pos;
public:
  static UListPos none;
  UListPos();
  void reset();
  int  getPos() const;
};

/* ==================================================== ======== ======= */

/** Logical "grouping" container.
 *
 * This class is used for grouping objects. it is a very lightweight
 * and general container. by opposition to the UBox class (and subclasses)
 * UGroup objects do not have Views and, thus, do not correspond to any
 * specific location on the Screen (there are just genuine containers)
 *
 * See:
 * - UGroup::~UGroup for <b>important notes about deletion</b>
 * - class UBox for info on how to combine boxes and interactors.
 * - class UCtrl as many methods are inherited from this class.
 */
class UGroup: public UCtrl {
public:
  static UStyle *style;   ///< corresponding Ubit UStyle

  UGroup(const UArgs& a = UArgs::none);
  /**< 
   * constructor; see: destructor ~UGroup(), class UArgs and <em>creator shortcut</em> <b>ugroup()</b>.
   */

  friend UGroup& ugroup(const UArgs& a = UArgs::none) {return *new UGroup(a);}
  /**<
   * <em>creator shortcut</em> that is equivalent to *new UGroup().
   * Note: watch the case: UGroup is a class while ugroup() is a function!
   */

  virtual ~UGroup();
  /**<
   * see <b>important notes</b> on <b>recursive deletion</b>.
   * All destructors <b>destroys the children</b> and descendants
   * of the object being deleted if:
   * - they do not have other (undeleted) parents, and
   * - they are not referenced by an uref (Ubit Reference), and
   * - they have been created in the heap (ie. by using the 'new' primivite
   *   or a 'creator shortcut' such as ugroup())
   *
   * <b>Advices:</b>
   * - avoid using the <b>delete</b> primivite and prefer the
   *   UGroup::remove() or UGroup::removeAll() functions
   *
   * - 'delete' does not update graphics and may produce unexpected 
   *   results or generate errors because objects:
   *   -# can be shared (= they can have several parents)
   *   -# can be created in the static memory or in the stack
   *   -# can be refrenced by uref's (Ubit References)
   *  deleting objects is illegal in the two later cases and will 
   *  generate an error or an exception.
   *
   * - See also: UGroup::remove(), UGroup::removeAll(), and uref
   */

  virtual const UStyle& getStyle(const UBox*) const {return makeStyle();}
  /**<
   * returns the contextual UStyle of this object.
   *  This virtual function calls the makeStyle() static function that
   *  was redefined for this specific class
   * - see also: UStyle and makeStyle() functions */

  static const UStyle& makeStyle();
  /**<
   * creates the Style of this object.
   * This static function is redefined by each class that derives from UGroup.
   * It is called by the virtual function UGroup::getStyle()
   * - note that certain classes have several makeStyle() functions (in which case
   *   they also have a specific getStyle() function)
   * - see also: UStyle and UGroup::getStyle() */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // child management 

  UGroup& addlist(const UArgs&);
  /**<
   * adds a <b>list</b> of children to the end of the object's child list.
   * Notes:
   * - this function is NOT virtual and returns the object
   * - this function updates graphics if the list contains UProp, UElem, 
   *   or UGroup objects
   * Exemple:
   * - UGroup& g = ugroup(); g.addlist(UColor::red + UFont::bold + "abcd"):
   */

  virtual void add(UBrick* child, int pos = -1, bool update = true);
  virtual void add(UBrick& child, int pos = -1, bool update = true);
  virtual void add(const char* child, int pos = -1, bool update = true);
  virtual void add(ULink* child, int pos = -1, bool update = true);
  virtual void add(ULink& child, int pos = -1, bool update = true);
  /**<
   * adds 'child' in the object's child list at this position.
   * Arguments:
   * - pos =  0  means "first child"
   * - pos = -1  (the default) means "after the last child"
   * - a non fatal error if produced if 'pos' is out of range
   * - the graphics display is updated if 'update_now' is true (you'll have 
   *   to call the update() method later otherwise: this can be useful for
   *   improving performances when many children must be added) 
   */

  virtual UBrick* remove(UBrick* child, bool auto_delete, bool update = true);
  virtual UBrick* remove(UBrick& child, bool auto_delete, bool update = true);
  /**<
   * removes 'child' OR <b>deletes</b> 'child' and its descendants.
   * Details:
   * - Step 1: 
   *   -#  the <b>first</b> occurence of 'child' is removed from the object's
   *       child list (other occurences are <b>not</b> removed if 'child' 
   *       appears several times in this list)
   *   -#  the graphics display is updated if 'update_now' is true (you'll
   *        have to call the update() method later otherwise: this can be 
   *        useful for improving performances when removing many children)
   *
   * - Step 2:
   *   -# if 'auto_delete' is 'false':
   *       'child' is <b>not</b> deleted and is returned by the function
   *   -#  if 'auto_delete' is 'true':
   *       'child' <b>and</b> its descendants are recursively deleted
   *       according to the rules described at UGroup::~UGroup
   *
   * - Returned value:
   *       the fct. returns null if 'child' was deleted and 'child' otherwise
   *
   * - Remarks:
   *   -# 'child' is never deleted if it appears several times in the 
   *       object's child list
   *   -# 'auto_delete' should generally be 'true' (as this is the only
   *       way to free unreferenced objects)
   *
   * - See also: UGroup::~UGroup and uref
   */

  virtual UBrick* remove(int pos, bool auto_delete, bool update = true);
  /**<
   * removes OR <b>deletes</b> the child at position 'pos' and its descendants.
   * Works in the same way as:  remove(UBrick&, bool, bool) except that:
   * - 'pos' =  0  means "first child" 
   * - 'pos' = -1  means "last child"
   * - a non fatal error if produced if 'pos' is out of range
   */

  virtual void removeAll(bool auto_delete, bool update = true);
  /**<
   * removes all children OR <b>deletes</b> all children and their descendants.
   * Works in the same way as:  remove(UBrick&, bool, bool).
   */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // access to children

  virtual int getChildCount() const;
  /**< returns the number of children */

  virtual UBrick** getChildren() const;
  virtual UBrick** getChildren(int& child_count) const;
  /**< 
   * returns a copy of the child list.
   * Returned value:
   * - returns null if there are no children.
   * - returns the child list if there is at least one child.
   *   This list must be destroyed by clients by calling delete[].
   *
   * Note: 
   * - Ubit objects can be added several times to the same parent.
   *   So, be careful when you delete children not to delete them twice
   *   (or more!) as some of them may appear several times in the child list.
   * See also:
   * - getChild() and other variants.
   */

  virtual int getChildren(std::vector<UBrick*>& child_vect) const;
  /**< 
   * returns a copy of the child list.
   * returns the number of children and intializes the vector of children.
   * Note: child_vect is not cleared : children are added to the 
   *       child_vect argument if its is not empty
   * See also: UBrick** getChildren() and getChild() (and other variants).
   */
  
  virtual UBrick* getChild(int pos) const;
  /**< 
   * returns the child at this position in the child list.
   * Arguments:
   * - 'pos' = 0 means "first child" and 'pos' = -1 means "last child"
   * - returns null if 'pos' is out of range (doesn't generate an error).
   *
   * Note:
   *   use getChild(UListPos&) or getChildren() is you need to access many 
   *   children in the child list: this will be more efficient than multiple 
   *   calls to getChild(int pos)
   */

  virtual int getChildPos(const UBrick& obj, int Nth = 0) const;
  virtual int getChildPos(const UBrick* obj, int Nth = 0) const;
  /**<
   * returns the position of the Nth occurence of 'obj' in the child list.
   * returns -1 if not found.
   * <p>'Nth' = 0 means "search first occurence", 'Nth' = 1 "search second
   * occurence", etc.   
   * <p>See also: getChild(const UBrick*, UListPos&) for mutiple calls.
   */

  virtual UBrick* getChild(UListPos&) const;
  /**<
   *  function for accessing all elements in the child list in an efficient way.
   *  Example:
   *  <pre><tt>
   *      UListPos lpos;            // scan children 
   *      while ((child = parent.getChild(lpos)))  {....}
   *
   *      lpos.reset();            // again, don't forget reset() !
   *      while ((child = parent.getChild(lpos)))  {....}
   *  </tt></pre>
   */

  virtual UBrick* getChild(UBrick* obj, UListPos& = UListPos::none) const;
  /**<
   *  searches for 'obj' in the child list.
   *  returns 'obj' if found, null otherwise.
   *  <p>Notes:
   *  - position in the list can be obtained though optional arg UListPos
   *  - UListPos can also be used for searching for multiple occurences
   *    of 'obj' in the child list.
   *
   *  Example:
   *  <pre><tt>
   *      UBrick* obj = ...;   // object to search for in the child list
   *      UListPos lpos;
   *      while (parent.getChild(obj, lpos))  {
   *          cout << "position: " << lpos.getPos() << endl;
   *      }
   *      lpos.reset();        // again, don't forget reset() !
   *      while ((child = parent.getChild(obj, lpos)))  {....}
   *  </tt></pre>   
   */

  virtual UBrick* getChild(UClassID class_id, UListPos& = UListPos::none) const;
  /**<
   *  searches objects with this class (or a subclass) in the child list.
   *  returns first child with this class or null if not found.
   *  <p>Notes:
   *  - can also retreive position in the list and multiple occurences: 
   *    see: getChild(const UBrick*, UListPos*) for details.
   */

  virtual UBrick* getChild(UListCall& predicate, bool& predicate_status, 
			   UListPos& = UListPos::none) const;
  /**<
   *  searches objects that verify this predicate in the child list.
   *  returns first appropriate child or null if not found.
   *  <p>Notes:
   *  - can also retreive position in the list and multiple occurences: 
   *     see: getChild(const UBrick*, UListPos&) for details.
   */
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual UBrick** getAttrs() const;
  virtual UBrick** getAttrs(int& child_count) const;
  ///< returns a copy of the attribute list (works in the same way as getChildren()).

  virtual int getAttrCount() const;
  ///< returns the number of attributes.

  virtual void addAttr(UBrick& child, int pos = -1, bool update = true);
  virtual void addAttr(ULink& child, int pos = -1, bool update = true);
  ///< adds an attribute (works in the same way as add()).

  virtual UBrick* removeAttr(UBrick& child, bool auto_delete, bool update = true);
  ///< removes an attribute (works in the same way as remove()).

  virtual void removeAllAttr(bool auto_delete, bool update = true);
  ///< removes all attributes (works in the same way as removeAll()).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // other methods

  virtual UStr copyText(bool recursive = true) const;
  virtual void copyTextTo(UStr&, bool recursive = true) const;
  /**<
   *  collates and returns the text that is enclosed in this object.
   *  'recursive' means that the text included in descendants is also collated.
   *  <p>Notes:
   *  - copyTextTo() is more efficient than copyText()
   *  - getTextSeparator() is used for separating enclosed children when
   *    'recursive' is true.
   */

  virtual const UStr* getTextSeparator() const;
  /**< 
   * text separator used by retreiveText() for separating enclosed children
   */

  virtual void show(bool = true);
  /**<
   * shows the object if argument is true; hides it if false.
   * see also: hide(), isShown(), isShowable()
   */

  virtual void hide() {show(false);}
  /**<
   * same as: show(false)
   */

  virtual void highlight(bool state);
  /**<
   * highlight the object (and brings it to the top if it is a window).
   */

  virtual void update();
  virtual void update(UUpdate upmode); 
  /**< 
   * updates this object's paint and/or layout.
   * <p>Arguments:
   * - no argument: updates layout then repaints
   * - 'upmode' argument : updates according to this argument. 
   *   see class UUpdate for details
   * <p>Note:
   * - update() must be called after add() or remove() if their last argument
   *   was set to 'false'
   */

  virtual void close(int status);
  /**< 
   * closes this object
   * Default behavior: hides the object (same as show(false)).
   * 
   * Can be redefined by subclasses for appropriate behaviors (such as object 
   * deletion, saving or freeing data, etc.)
   * 
   * Note: status is only meaningful for UWin subclasses.
   * <p>See also: UBox::closeWin(), UWin::close()
   */

  static void  closeWin(class UEvent&, int status);
  virtual void closeWin(int status);
  /**<
   * closes the first window (UDialog, UMenu...) that contains this object.
   * Notes:
   * - status should be >= 0 (see UWin::lock())
   * - calls UWin::closeWin()
   * - called by callback object: ucloseWin()
   */

  friend UCall& ucloseWin(int status = 0)
  {return ucall(status, &UGroup::closeWin);}
  /**<
   * callback that closes the first window (UDialog, UMenu...) that contains this object.
   * same effect as: UGroup::closeWin()
   */
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  enum ChildList {AttrList, ElemList};
  enum RemoveMode {RemFromParents=-1, NoDel=0, AutoDel=1};

  virtual void 
  addImpl(ChildList, UBrick* child, class ULink* childlink, int pos, 
	  bool update, bool* should_update);
  virtual UBrick* 
  removeImpl(ChildList, UBrick* child, class ULink* prevlink, 
	     RemoveMode, bool update, bool* should_update);
  class ULink* 
  getChildImpl(ChildList, const UBrick* child, int nth, 
	       class ULink*& prevlink, int* pos) const;
  class ULink* 
  getChildImpl(ChildList, int pos, class ULink*& prevlink) const;
 /**<
   * these methods are called by add(), remove(), getChild()
   * and 'delete'. They can can be freely redefined by subclasses. 
   * Note however its is generally necessary to redefine the four of them 
   * simultaneouly as these methods call each other */

  virtual void fire(class UEvent&, const class UOn&) const; 
  ///< [impl] fires callback functions and callback methods.

  virtual void cleanup();

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  friend class UBrick;
  friend class UWin;

  virtual class UGroup* groupCast() {return this;}

  class ULink* getAttrLinks() const;
  class ULink* getChildLinks() const;
  ///< [impl] returns internal child links.

  virtual UGroup* getBrowseGroup() {return null;}
  ///< [impl] this fct. makes this object an implicit browsing group
  ///< if it returns the object default/ NOT a browsing group)

  virtual void init(ULink *selflink, UView *ancestor_view);
  virtual void initAddedChild(UGroup*, ULink *childlink);
  virtual void updateView(UEvent&, UView*, const UUpdate&);

protected:
  static class UChain firingList; //objects whose callbacks are now being fired
  class UChain children;	  // child list

  virtual int   getTextLength(bool recursive) const;
  virtual char* getTextData(char *ptr, bool recursive) const;
  ///<  [impl] intermediate functions for retrieveText()

#endif
};


#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
