/* ==================================================== ======== ======= *
 *
 *  uima.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (Subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uima_hh
#define	_uima_hh
//pragma ident	"@(#)uima.hh	ubit:2.8.0"
#include <uelem.hh>

/** Image Object.
 * Notes: 
 * - supported formats depend on which packages where linked to the Ubit lib
 *   (XPM and GIF formats are supported by default)
 * - a pathname prefix is implicitely added to the file names except if
 *   they start by / or .
 * - file names can start by a path variable (eg. "$mypath/whatever/file.gif")
 *   see UAppli.setImaPath() for details
 * - background transparency (ie. image shapes) is not supported by UIma
 *   but is supported by subclass UPix for XPM images
 */
class UIma: public UElem {
public:
  UIma(const UStr& filename);
  UIma(const char* filename = null);
  UIma(const char** xpm_data, u_modes = 0);
  /**<
   * creates a new image and specifies its content source for postponed loading.
   * The argument can specify a file name or XPM compiled data. By default,
   * the data is loaded the first time this image becomes visible.
   * <p>See: 
   * - set(), read() and realize() (for details on postponed or immediate loading)
   * - class UIma (for supported file formats and the syntax of file names)
   * <p>Note: 
   * - 'xpm_data' is <b>not</b> copied internally and must <b>not</b> be freed */

  friend UIma& uima(const UStr& filename)  {return *new UIma(filename);}
  friend UIma& uima(const char* filename)  {return *new UIma(filename);}
  friend UIma& uima(const char** xpm_data) {return *new UIma(xpm_data);}
  ///< creator shortcuts (see constructors UIma::UIma() for details)

  virtual ~UIma();

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void set(const UStr& filename);
  virtual void set(const char* filename);
  virtual void set(const char** xpm_data);
  /**<
   * postponed loading of an image file (or compiled XPM data).
   * Loads image file (or image data) and updates the display when this 
   * image becomes visible (the loading takes places immediately only
   * if this image is already visible).
   * <p>See also: 
   * - read() and realize() (for immediate loading)
   * - class UIma (for supported file formats and the syntax of file names)
   * <p>Notes: 
   * - 'xpm_data' is <b>not</b> copied internally and must <b>not</b> be freed
   * - images that are not shown are never loaded (this makes it possible
   *   to specify large sets of images that may potentially be used without
   *   having to care about memory use */

  virtual int read();
  virtual int readFile(const UStr& filename);
  virtual int readFile(const char* filename);
  virtual int readData(const char** xpm_data);
  /**<
   * immediate loading of an image file (or compiled XPM data).
   * Loads image file (or image data) immediately. The display is updated
   * if the image is already visible.
   * <p>See also: 
   * - getStatus() : returns the loading status (see UFilestat)
   * - set() (for postponed loading)
   * - class UIma (for supported file formats and the syntax of file names)
   * <p>Notes:
   * - 'xpm_data' is <b>not</b> copied internally and must <b>not</b> be freed 
   * - this function is equivalent to: set() + realize() + update() */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void copy(const UIma& ima2);
  /**< 
   * copies image content from ima2.
   * No action performed is ima2 is not realized (ie. not loaded) */

  void copyResized(const UIma& ima2, u_dim width, u_dim height);
  /**< 
   * copies image content from ima2 and rescales to size: width x height.
   * see: copy(const UIma&) for details */

  void copyRescaled(const UIma&, float xscale, float yscale);
   /**< 
   * copies image content from ima2 and rescales according to scaling factors.
   * see: copy(const UIma&) for details */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  const char* getNameChars() const;
  ///< returns the image name (if any, null otherwise)

  int getStatus();
  ///< returns the loading status of this image (see UFilestat)

  bool isRealized() const;
  ///< returns true if the image has been loaded and realized (see: realize()).

  bool isShaped() const;
  bool isTransparent() const {return isShaped();}
  ///< return true if the image background is shaped (so called "transparent").

  virtual u_dim getHeight() const;
  virtual u_dim getWidth()  const;
  ///< returns the image size (unknown if the image has not been loaded!).

  virtual void update(); 
  ///< update parents' views.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  int realize(class UDisp& disp, bool force_reload = true) const;
  /**< 
   * [impl] allocates physical resources (and loads the image).
   * this function is automatically called when this image is displayed
   * for the first time. There no need to call it in client code except
   * for loading images in advance (see also the load() function).
   * details:
   * - returns the loading status  (see UFilestat)
   * - the 'disp' arg. can be the UAppli (this is the default)
   * - the 'force_reload' arg. specifies if the file must be reloaded:
   *   # if false, previously loaded files are not reloaded
   *   # if true files are always reloaded (useful if files have changed) */

  class UNatIma* getNatIma() const {return natimas;}
  ///< [impl] returns internal implementation.

#ifndef NO_DOC
  friend class UBox;
  friend class UNatIma;
  friend class UPix;

  void setImpl(const char* fname);
  void setImpl(const char** xpm_data);
  virtual void cleanCache();
  virtual void getSize(UContext*, u_dim *w, u_dim *h) const;
  virtual void paint(UWinGraph&, UContext*, const URegion &r) const;

protected:
  mutable class UNatIma *natimas;
  char *name;
  const char **data;
  mutable signed char stat;  // can have negative values
  mutable bool read_from_file, read_from_data, show_unknown_ima;
  static class UNatIma
    *findImaInCache(class UNatDisp*, class UNatIma *natimas, int lscale),
    *addImaInCache(class UNatDisp*, class UNatIma *natimas, int lscale);
#endif
};


#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
