/* ==================================================== ======== ======= *
 *
 *  uprop.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uprop_hh
#define	_uprop_hh
//pragma ident	"@(#)uprop.hh	ubit:2.11.0"
#include <unumber.hh>

/** Base class for Graphical Properties.
 * Notes: 
 * - the (inherited) onChange() method adds a callback that is activated
 *   when the object's value changes.
 * - the 'UOn::propChange' callbacks of the object's parents are also 
 *   activated (but after the callbacks of this object)
 */
class UProp: public UBrick {
  friend class UBox;
public:
  UProp(u_modes b_modes = 0) : UBrick(b_modes) {}
  ///< constructor.

  virtual void onChange(UCall&);
  /**< adds a callback that is activated when the objects' value changes.
   */

  virtual void update() = 0;
  ///< updates graphics.

  virtual class UProp* propCast() {return this;}

  virtual void putProp(class UContext*, class UCtrl*) = 0;
  ///< [impl] changes corresponding value in the UContext

protected:
  virtual void changed(bool update_now);
  /**< [impl] called when object's content is changed.
   * This function:
   * - updates grahics (if arg is true)
   * - then fires object's UOn::change callbacks
   * - then fires parents' UOn::changeProp callbacks
   */
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Scale property.
 *  Scale properties can be cascaded in the instance graph.
 *  set*() and get*() methods are inherited from Uintg.
 */
class UScale : public UIntg, public UProp {
public:
  UScale(const UIntg& value = 0);

  friend UScale& uscale(const UIntg& val = 0) {return *new UScale(val);}
  ///< larger than normal size if 'value' is positive; smaller if negative.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void incr(int _v)          {UIntg::incr(_v);}
  virtual void incr(const UIntg& _v) {UIntg::incr(_v);}

  virtual void decr(int _v)          {UIntg::decr(_v);}
  virtual void decr(const UIntg& _v) {UIntg::decr(_v);}

  float getXScale();
  float getYScale();

  virtual void onChange(UCall&);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void update();
  virtual void changed(bool update_now);
  virtual void putProp(class UContext*, class UCtrl*);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Alpha blending property.
 *  Note: name subjet to change.
 */
class UAlpha : public UProp {
  float value;
public:
  UAlpha(float value = 1.0);
  friend UAlpha& ualpha(float val = 1.0) {return *new UAlpha(val);}
  ///< value must be in range [0.0, 1.0] : 1.0 : opaque object; 0.0 : fully transparent object.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(float value);
  void set(float value, bool update_now);
  ///< changes alpha value. must be in interval [0.0, 1.0].

  float get() const {return value;}
  ///< returns current alpha value.

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Background property.
 * See also: UBgcolor
 */
class UBackground : public UProp {
  const class UColor  *bgcolor;
  const class UIma    *bgima;
  const class UHalign *halign;
  const class UValign *valign;
public:
  UBackground();
  UBackground(const UIma& bgima);
  UBackground(const UColor& bgcolor);

  friend UBackground& ubackground(const UColor& bgcolor);
  friend UBackground& ubackground(const UIma& bgima);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(const UColor& bgcolor);
  ///< sets an uniform background color.

  void set(const UIma& bgima);
  /**< sets a tiled background from an image.
   * Note: the background image can be a transparent UPix */

  void set(const UIma& bgima, const UColor& bgcolor);
  /**< sets a tiled background with a background color.
   * Note: useful if the image is transparent
   * or if the image is centered instead of being tiled */

  void setLayout(const UHalign&, const UValign&);
  /**< if null, the bg image is tiled in this direction
   * if &UHalign::center, it is centered in the horizontal direction, etc.*/

  const class UColor*  getColor()  const {return bgcolor;}
  const class UIma*    getIma()    const {return bgima;}
  const class UHalign* getHalign() const {return halign;}
  const class UValign* getValign() const {return valign;}

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */
/*
class UShape : public UProp {
  friend class UView;
  const UPix *pix;
public:
  // The shape of the component will adapt to the shape of the UPix argument
  UShape(const UPix&);
  friend UShape& ushape(UPix&);

  void set(const UPix*);
  void set(const UPix&);

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};
*/
/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Flag Definition property.
 *  See: UFlag.
 */
class UFlagDef : public UProp {
  const class UFlag* flag;
public:
  UFlagDef();
  UFlagDef(const UFlag&);
  friend UFlagDef& uflagDef(const class UFlag& f) {return *new UFlagDef(f);}

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(const UFlag&);
  //< do: set(UFlag::none) to unset a flag definition

  const UFlag* getFlag() {return flag;}

  virtual void update();
  virtual void putProp(UContext*, class UCtrl*);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Position property.
 *  Locates an UBox (or subclass) object at an arbitrary position.
 *  - coordinates are relative to parent.
 *  - !Warning: does not work with hardwin UWin objects (use UWin::move())
 */
class UPos : public UProp {
  u_pos x, y;
public:
  UPos();
  UPos(u_pos x, u_pos y);
  friend UPos& upos(u_pos x, u_pos y) {return *new UPos(x, y);}

  virtual ~UPos() {cleanup();}  // necessaire car removingFrom specifique

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(u_pos _x, u_pos _y);
  void set(u_pos _x, u_pos _y, bool update_now);
  void setX(u_pos _x) {set(_x, y, true);}
  void setY(u_pos _y) {set(x, _y, true);}

  u_pos getX() const {return x;}
  u_pos getY() const {return y;}

  virtual void update();

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  virtual void putProp(class UContext*, class UCtrl*);

  virtual void addingTo(class ULink *selflink, UGroup *parent);
  virtual void removingFrom(class ULink *selflink, UGroup *parent);
  ///< NOTE that this function require a specific destructor.
#endif
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Width property.
 * Usage:
 * - uwidth(int) with int >= 0  : size fixed by argument.
 *   horizontal size won't change except if the box is uhflex()
 * - uwidth(UWidth::KEEP_SIZE)   : keep initial size.
 *   size is computed automatically the first time, then does not change.
 * - uwidth(UWidth::AUTO_RESIZE) : size automatically adapts
 *   size is re-computed automatically each time 
 */
class UWidth : public UProp {
  u_dim value;
public:
  static const u_dim AUTO_RESIZE, KEEP_SIZE;

  UWidth(u_dim value = 0);
  friend UWidth& uwidth(u_dim value);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(u_dim);
  u_dim get() const {return value;}

  UWidth& operator=(u_dim _val)   {set(_val); return *this;}
  UWidth& operator=(UWidth& _val) {set(_val.get()); return *this;}

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};


/** Height property.
 * Usage: see UWidth.
 */
class UHeight : public UProp {
  u_dim value;
public:
  static const u_dim AUTO_RESIZE, KEEP_SIZE;

  UHeight(u_dim value = 0);
  friend UHeight& uheight(u_dim value);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void  set(u_dim);
  u_dim get() const {return value;}

  UHeight& operator=(u_dim _val)    {set(_val); return *this;}
  UHeight& operator=(UHeight& _val) {set(_val.get()); return *this;}

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Orientation property.
 *  Can be: UOrient::vertical or  UOrient::horizontal
 */
class UOrient : public UProp {
  char value;
public:
  enum {VERTICAL, HORIZONTAL, INHERIT};
  static UOrient vertical, horizontal, inherit;

  UOrient(char value = VERTICAL, u_modes = 0);
  UOrient(const UOrient&);
  friend UOrient& uorient(const UOrient&);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(const UOrient&);
  char get() const {return value;}

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Vertical Alignment property.
 *  Can be: top, bottom, center, flex ("flex" means "flexible object")
 *  - rather use functions: utop(), ubottom(), uvcenter(), uvflex() 
 *    that are equivalent but improve readability
 */
class UValign : public UProp {
  char value;
public:
  enum {TOP, BOTTOM, FLEX, CENTER};
  static UValign top, bottom, flex, center;

  UValign(char value = TOP, u_modes = 0);
  UValign(const UValign&);
  friend UValign& uvalign(const UValign&);

  friend UValign& utop()     {return UValign::top;}
  friend UValign& ubottom()  {return UValign::bottom;}
  friend UValign& uvcenter() {return UValign::center;}
  friend UValign& uvflex()   {return UValign::flex;}

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(const UValign&);
  char get() const {return value;}

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};


/** Horizontal Alignment property.
 *  Can be: left, right, center, flex ("flex" means "flexible object")
 *  - rather use functions: uleft(), uright(), uhcenter(), uhflex() 
 *    that are equivalent but improve readability
 */
class UHalign : public UProp {
  char value;
public:
  enum {LEFT, RIGHT, FLEX, CENTER};
  static UHalign left, right, flex, center;

  UHalign(char value = LEFT, u_modes = 0);
  UHalign(const UHalign&);
  friend UHalign& uhalign(const UHalign&);

  friend UHalign& uleft()    {return UHalign::left;}
  friend UHalign& uright()   {return UHalign::right;}
  friend UHalign& uhcenter() {return UHalign::center;}
  friend UHalign& uhflex()   {return UHalign::flex;}

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(const UHalign&);
  char get() const {return value;}

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Vertical Spacing property.
 */
class UVspacing : public UProp {
  int value;

public:
  UVspacing(int = 0);
  friend UVspacing& uvspacing(int);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(int);
  int get() const {return value;}

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};


/** Horizontal Spacing property.
 */
class UHspacing : public UProp {
  int value;

public:
  UHspacing(int = 0);
  friend UHspacing& uhspacing(int);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(int);
  int get() const {return value;}

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/** Vertical Margin property.
 */
class UVmargin : public UProp {
  int value;

public:
  UVmargin(int = 0);
  friend UVmargin& uvmargin(int);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(int);
  int get() const {return value;}

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};


/** Horizontal Margin property.
 */
class UHmargin : public UProp {
  int value;

public:
  UHmargin(int = 0);
  friend UHmargin& uhmargin(int);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  void set(int);
  int get() const {return value;}

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/* Tool Tip.
 */
class UTip: public UProp {
  uref<UStr> value;
public:

  UTip(const char* value);
  ///< contructor (value is duplicated internally).

  UTip(UStr& value);
  /**< contructor (value is NOT duplicated internally and must NOT be deleted)
   * Note: 'value' can be shared and directly modified by clients.
   * It should not be deleted nor allocated in the stack.
   * It will be automatically deleted when not pointed by any uref.
   */

  friend UTip& utip(const char* _value) {return *(new UTip(_value));}
  friend UTip& utip(UStr& _value)       {return *(new UTip(_value));}
  ///< creator shortcuts.

  virtual void set(const UStr& value);
  virtual const UStr& get() const {return *value;}

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*) {}
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/* Window Title.
 */
class UTitle: public UProp {
  uref<UStr> value;

public:
  UTitle(const char* value);
  ///< contructor (value is duplicated internally).

  UTitle(UStr& value);
  /**< contructor (value is NOT duplicated internally and must NOT be deleted)
   * Note: 'value' can be shared and directly modified by clients.
   * It should not be deleted nor allocated in the stack.
   * It will be automatically deleted when not pointed by any uref.
   */

  friend UTitle& utitle(const char* _value) {return *new UTitle(_value);}
  friend UTitle& utitle(UStr& _value)       {return *new UTitle(_value);}
  ///< creator shortcuts.

  virtual void set(const UStr& value);
  virtual const UStr& get() const {return *value;}

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*) {}
};

/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */

/* Comment (not displayed)
 */
class UComment: public UProp {
  char *value;

public:
  UComment(const char* = null);
  friend UComment& ucomment(const char *s);

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void set(const char*);
  virtual const char* get() const {return value;}

  virtual void update();
  virtual void putProp(class UContext*, class UCtrl*);
};

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
