/* ==================================================== ======== ======= *
 *
 *  uselect.hh
 *  Ubit Project [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _uselect_hh
#define	_uselect_hh
//pragma ident	"@(#)uselect.hh	ubit:2.11.0"

/** Exclusive selector for selecting boxes (eg. UCheckbox, UButton, etc.).
 *  <p>This selector must be a child of the items it selects.
 *  These items are automatically made selectable (for instance an UButton
 *  will become selectable if an USelect is added to it as a child). 
 *  <pre>
 *  </pre>
 */
class USelect : public UProp {
public:
  USelect();
  /**< constructor.
   * Notes: 
   * - this selector must be a child of the items it selects
   * - selection is exclusive
   * - no item is initially selected */

  friend USelect& uselect() {return *(new USelect());}
  ///< creator shortcut (see constructor for details).

  virtual ~USelect();
  ///< destructor.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual UGroup* getSelectedItem() const;
  ///< returns the selected item (null if none selected)

  virtual int getSelectedIndex() const;
  ///< returns the index of the selected item (-1 if nothing selected; 0 if first item selected).

  virtual void setSelectedItem(UGroup*);
  virtual void setSelectedItem(UGroup&);
  ///< selects this item (deselects all items if null given as an arg).

  virtual void setSelectedIndex(int);
  virtual void select(int);
  ///< selects the item at this index in the list (0 is first item, -1 last item).

  virtual void unselect();
  ///< unselects all items

  virtual void setCanUnselectMode(bool);
  virtual bool isCanUnselectMode() const;
  ///< clicking on a selected item will unselect it if true (default is false).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void update();
  ///< updates graphics (this function must be called when adding or removing items).

  virtual void changed(bool update_now);
  /**< called when the selection is changed.
   * This function:
   * - updates grahics (if arg is true)
   * - first fires object's UOn::change and UOn::select callbacks
   * - then fires parents' UOn::childSelect callbacks
   */

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  virtual void putProp(UContext*, class UCtrl*);

  virtual void addingTo(ULink *selflink, UGroup *parent);
  // NB: removingFrom() requires a destructor to be defined
  virtual void removingFrom(ULink *selflink, UGroup *parent);

protected:
  int   index;
  bool  can_unselect_mode;
  uref<UCall> calls;   // callbacks
  void  itemSelectChanged(UEvent&);
#endif
};


/* ==================================================== ======== ======= */
/* ==================================================== ======== ======= */
/* ==================================================== [Elc:02] ======= */
/* ==================================================== ======== ======= */
/* ==================================================== ======== ======= */

/** Exclusive selector for selecting boxes (eg. UCheckbox, UButton, etc.).
 *  <p>This selector must be a child of the items it selects.
 *  These items are automatically made selectable (for instance an UButton
 *  will becone selectable if an USelect is added to it as a child). 
 *  <pre>
 *  Exemple:
 *    URadioSelect& sel = uradioSelect();
 *    ubox(ubutton(sel + "One") + ucheckbox(sel + "Two") + utextbox(sel + "Three")) 
 *  </pre>
 */
class URadioSelect : public UProp {
public:
  URadioSelect();
  /**< constructor.
   * Notes: 
   * - this selector must be a child of the items it selects
   * - selection is exclusive
   * - no item is initially selected */

  friend URadioSelect& uradioSelect() {return *(new URadioSelect());}
  ///< creator shortcut (see constructor for details).

  virtual ~URadioSelect();
  ///< destructor.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual UGroup* getSelectedItem() const;
  ///< returns the selected item (null if none selected)

  virtual int getSelectedIndex() const;
  ///< returns the index of the selected item (-1 if nothing selected; 0 if first item selected).

  virtual void setSelectedItem(UGroup*);
  virtual void setSelectedItem(UGroup&);
  ///< selects this item (deselects all items if null given as an arg).

  virtual void setSelectedIndex(int);
  virtual void select(int);
  ///< selects the item at this index in the list (0 is first item, -1 last item).

  virtual void unselect();
  ///< unselects all items

  virtual void setCanUnselectMode(bool);
  virtual bool isCanUnselectMode() const;
  ///< clicking on a selected item will unselect it if true (default is false).

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void update();
  ///< updates graphics (this function must be called when adding or removing items).

  virtual void changed(bool update_now);
  /**< called when the selection is changed.
   * This function:
   * - updates grahics (if arg is true)
   * - first fires object's UOn::change and UOn::select callbacks
   * - then fires parents' UOn::childSelect callbacks
   */

#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  virtual void putProp(UContext*, class UCtrl*);

  virtual void addingTo(ULink *selflink, UGroup *parent);
  // NB: removingFrom() requires a destructor to be defined
  virtual void removingFrom(ULink *selflink, UGroup *parent);

protected:
  int   index;
  bool  can_unselect_mode;
  uref<UCall> item_select_changed;
  void  itemSelectChanged(UEvent&);
#endif
};


#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
