/* ==================================================== ======== ======= *
 *
 *  ustr.hh
 *  Ubit Project  [Elc][2002]
 *  Author: Eric Lecolinet
 *
 *  Part of the Ubit Toolkit: A Brick Construction Game Model for Creating GUIs
 *
 *  (C) 1999-2002 Eric Lecolinet @ ENST Paris
 *  WWW: http://www.enst.fr/~elc/ubit   Email: elc@enst.fr (subject: ubit)
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************
 *
 * ==================================================== [Elc:02] ======= *
 * ==================================================== ======== ======= */

#ifndef _ustr_hh
#define	_ustr_hh
//pragma ident	"@(#)ustr.hh	ubit:2.11.0"
#include <uelem.hh>

/** Active String class.
 * Notes: 
 * - the (inherited) onChange() method adds a callback that is activated
 *   when the object's value changes.
 * - the 'UOn::elemChange' and 'UOn::strChange' callbacks of the object's 
 *   parents are also activated (but after the callbacks of this object)
 */
class UStr: public UElem {
  char* s;
  int len;
public:
  static UStr newline, goBOL;
  ///< goBOL = go to Beginning Of Line (if needed)

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  UStr(const char* = null);
  UStr(const std::string&);
  UStr(const UStr&);
  ///< constructors: arg is a C, C++ or Ubit string; data is copied internally so that arg can be freely destroyed.

  friend UStr& ustr(const char* _s = null)  {return *new UStr(_s);}
  friend UStr& ustr(const std::string& _s)  {return *new UStr(_s);}
  friend UStr& ustr(const UStr& _s)         {return *new UStr(_s);}
  ///< creator shortcuts (same as *new UStr()).

  virtual ~UStr();
  ///< destructor.

  virtual void update();
  ///< updates grahics.
  
  virtual void changed(bool update_now);
  /**< [impl] called when object's content is changed.
   * This function:
   * - updates grahics (if arg is true)
   * - then fires object's UOn::change callbacks
   * - then fires parents' UOn::changeProp callbacks
   * - then fires parents' UOn::changeStr callbacks
   */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  UStr& operator=(const UStr& s2)         {set(s2); return *this;}
  UStr& operator=(const char* s2)         {set(s2); return *this;}
  UStr& operator=(const std::string& s2)  {set(s2); return *this;}
  UStr& operator=(const UIntg& v2)        {set(v2); return *this;}
  UStr& operator=(const UFloat& v2)       {set(v2); return *this;}
  ///< s1 = v2 : copies the value of v2 to s1 (same as set() but not virtual).

  UStr& operator&=(const UStr& s2)        {append(s2); return *this;}
  UStr& operator&=(const char* s2)        {append(s2); return *this;}
  UStr& operator&=(const std::string& s2) {append(s2); return *this;}
  UStr& operator&=(const UIntg& v2)       {append(v2); return *this;}
  UStr& operator&=(const UFloat& v2)      {append(v2); return *this;}
  /**< s1 &= s2 : appends the value of v2 to s1 (same as append() but not virtual).
   * Note: we use &= rather than += because the + operator is used for 
   * making argument lists.
   */

   UStr operator&(const UStr& s2) const;
   UStr operator&(const char* s2) const;
   UStr operator&(const std::string& s2) const;
   UStr operator&(const UIntg& v2) const;
   UStr operator&(const UFloat& v2) const;
  /**< s0 = s1 & s2 : concatenates values (does not change arguments).
   * Note: we use & rather than + because the + operator is used for 
   * making argument lists.
   */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void set(const char* s2);
  virtual void set(const UStr& s2);
  virtual void set(const std::string& s2);
  virtual void set(const UIntg& v2);
  virtual void set(const UFloat& v2);
  ///< s1.set(v2) : copies the value of v2 to s1 (same as operator = but virtual).

  virtual void setInt(int v2);
  virtual void setLong(long v2);
  virtual void setFloat(float v2);
  virtual void setDouble(double v2);
  ///< s1.set(v2) : copies the value of v2 to s1 (after appropriate conversion).

  friend std::ostream& operator<<(std::ostream&, const UStr&);
  ///< prints string on ostream, for instance: cout << s.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual bool empty() const {return (len == 0);}

  virtual bool equals(const char*) const;
  virtual bool equals(const std::string&) const;
  virtual bool equals(const UStr&) const;
  ///< returns true if string contents are the same.

  virtual int compareTo(const char* s2) const;
  virtual int compareTo(const UStr& s2) const;
  ///< strings are compared in the lexicographical order.

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  int length() const {return len;}
  ///< returns the length of the string.

  const char* chars() const {return s;}
  /**< 
   *  returns a C string that should not be changed nor deleted.
   *  See also: getChars() if you need a string duplicate */

  virtual char* getChars() const;
  virtual char* getChars(int from_pos, int nbchars) const;
  /**< 
   *  returns a C string that is a COPY of the UStr content.
   *  The second variant returns 'nbchars' characters from offset 'from_pos'
   *  (the result is null or truncated if out of bounds).
   *  The returned data should be freed by client by calling delete[].
   *  See also: chars() if you don't need to modify the returned C string.
   */

  virtual std::string getString() const;
  virtual std::string getString(int from_pos, int nbchars) const;
  /**< 
   *  returns a C++ string that is a COPY of the UStr content.
   *  The second variant returns 'nbchars' characters from offset 'from_pos'
   *  (the result is null or truncated if out of bounds).
   *  See also: copyTo(), appendTo(), insertInto(). 
   */
 
  virtual void copyTo(std::string&) const;
  virtual void copyTo(std::string&, int from_pos, int nbchars) const;
  virtual void appendTo(std::string&) const;
  /**< 
   *  copies or appends the UStr content to a C++ string.
   *  The second variant returns 'nbchars' characters from offset 'from_pos'
   *  (the result is null or truncated if out of bounds).
   *  Note that copyTo() is similar to but more efficient than getString().
   *  See also: getString(), appendTo(), insertInto(). 
   */

  virtual void insertInto(std::string& str, int to_pos) const;
  /**< 
   *  inserts the UStr content into a C++ string at this position.
   *  to_pos = -1 means "end of string" (same as append()).
   *  See also: getString(), copyTo(), appendTo(). 
   */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual int charAt(int pos) const;
  ///< returns character at offset 'pos' (-1 = last pos) and 0 if invalid pos.

  virtual bool setCharAt(int pos, int newchar);
  /**< changes character at offset 'pos' (-1 means last char of string).
   * notes:
   * - returns false if char can't be inserted (wrong position or invalid newchar)
   * - UOn::change callbacks are activated (see method onChange()) */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  virtual void append(const UStr& s2);
  virtual void append(const char* s2);
  virtual void append(const std::string& s2);
  virtual void append(const UIntg& v2);
  virtual void append(const UFloat& v2);
  ///< appends s2 to the end of this string (same as insert() with 'to_pos' = -1).

  virtual void insert(int to_pos, int newchar);
  virtual void insert(int to_pos, const UStr& s2);
  virtual void insert(int to_pos, const char* s2);
  virtual void insert(int to_pos, const UStr& s2, int from_pos, int from_nbchars);
  virtual void insert(int to_pos, const char* s2, int from_pos, int from_nbchars);
  ///< inserts s2 at offset 'to_pos' in this string (0 = beginning of string; -1 = end of string).

  virtual void remove(int pos, int nbchars);
  ///< removes 'nbchars' characters from offset 'pos' (0 = beginning of string; -1 = last char of string).

  virtual void replace(int pos, int nbchars, const UStr& s2);
  virtual void replace(int pos, int nbchars, const char* s2);
  ///< replaces 'nbchars' characters from offset 'pos' by 's2' (combines insert() and remove()).

  virtual void strip(bool strip_beginning = true, bool strip_end = true);
  ///< removes white spaces, tabs and newlines at the beginning and end of string.

  virtual UStr split(int pos);
  ///< splits the string: removes contents from offset 'pos' and returns the remaining chars */

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // file name manipulation

  virtual UStr getFileName(bool with_suffix = true) const;
  ///< returns the file name of a file path (= without the directory); removes the suffix if arg is false.

  virtual UStr getFileSuffix() const;
  ///< returns the suffix of a file path (= the end of the pathname after the dot)

  virtual UStr getFileDir(bool with_final_slash = true) const;
  ///< returns the directory of a file path (= without the file name).

  virtual long getFileType() const;
  ///< returns the Unix file type (0 if file does not exist)

  virtual bool isFileFound(bool is_regular_file = true) const;
  ///< returns true if file exists (also checks that this is a regular file if arg is true)

  virtual int readFile(const UStr& filename);
  virtual int readFile(const char* filename);
  ///< reads this file into the UStr and returns the loading status (see UFilestat).


#ifndef NO_DOC
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation

  UStr(const char*, u_modes bmodes);

  virtual class UStr* strCast() {return this;}
  virtual void syncVals(char*,int);

  virtual void addingTo(class ULink *selflink, UGroup *parent);
  virtual void removingFrom(class ULink *selflink, UGroup *parent);
  ///< NOTE that this function require a specific destructor.

  virtual bool checkFormat(int pos, int newchar);
  virtual bool checkFormat(int pos, const char*);
  ///< checks input conformance to format model (if any).

  virtual bool setImpl(const char*, int len, bool call_callbacks);
  virtual bool insertImpl(int to_pos, char newchar, bool upd);
  virtual bool insertImpl(int to_pos, const char* s2, 
			  int from_pos, int from_len, bool upd);
  virtual bool replaceImpl(int pos, int nbchars, const char* s2, bool upd);

  // prototype for non-warped text (UView)
  virtual void getSize(class UContext*, u_dim *w, u_dim *h) const;

  // prototype for warped text (UFlowView)
  virtual void getSize(class UContext*, u_dim *w, u_dim *h, 
		       u_dim available_width, int offset, 
		       int *sublen, int *change_line) const;

  // prototype for non-warped text (UView)
  virtual void paint(class UWinGraph&, class UContext*, const URegion &r) const;

  // prototype for warped text (UFlowView)
  virtual void paint(class UWinGraph&, class UContext*, const URegion &r,
		     int offset, int cellen) const;
#endif
};


/* === useful functions =============================== ======== ======= */

char *u_strdup(const char *s1);
///< creates a duplicate of 's1' (s1 can be null) NB: this fct is similar to strdup().

char *u_strdupcat(const char *s1, const char *s2);
///< creates a duplicate of 's1' + 's2' (arguments can be null).

char *u_strdupcat(const char *s1, char sep, const char *s2);
///< creates a duplicate of 's1' + 'sep' + 's2' ('sep' is not added if either 's1' or 's2' is null).

char *u_strext(char *pathname);
const char *u_strext(const char *pathname);
/**< finds the file suffix in a pathname.
 * - note: the returned value is null or points inside the original string
 *  (its is not duplicated and thus, can't be freed) */

void u_strparse(char* path_name, int path_len,
		char*& dir_name, int& dir_len,
		char*& base_name, int& base_len,
		char*& suffix, int& suffix_len);
/**< decomposes a path name into dir name, base name and suffix components.
 * the returned pointers points inside the path name and should not be freed.
 */

#endif
/* ==================================================== [TheEnd] ======= */
/* ==================================================== [Elc:02] ======= */
